"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def restore(
    hub,
    ctx,
    db_cluster_identifier: str,
    source_db_cluster_identifier: str,
    restore_type: str = None,
    restore_to_time: str = None,
    use_latest_restorable_time: bool = None,
    port: int = None,
    db_subnet_group_name: str = None,
    option_group_name: str = None,
    vpc_security_group_ids: List = None,
    tags: List = None,
    kms_key_id: str = None,
    enable_iam_database_authentication: bool = None,
    enable_cloudwatch_logs_exports: List = None,
    db_cluster_parameter_group_name: str = None,
    deletion_protection: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Restores a DB cluster to an arbitrary point in time. Users can restore to any point in time before LatestRestorableTime for up to BackupRetentionPeriod days. The target DB cluster is created from the source DB cluster with the same configuration as the original DB cluster, except that the new DB cluster is created with the default DB security group.  This action only restores the DB cluster, not the DB instances for that DB cluster. You must invoke the CreateDBInstance action to create DB instances for the restored DB cluster, specifying the identifier of the restored DB cluster in DBClusterIdentifier. You can create DB instances only after the RestoreDBClusterToPointInTime action has completed and the DB cluster is available.

    Args:
        db_cluster_identifier(str): The name of the new DB cluster to be created. Constraints:   Must contain from 1 to 63 letters, numbers, or hyphens   First character must be a letter   Cannot end with a hyphen or contain two consecutive hyphens  .
        restore_type(str, optional): The type of restore to be performed. You can specify one of the following values:    full-copy - The new DB cluster is restored as a full copy of the source DB cluster.    copy-on-write - The new DB cluster is restored as a clone of the source DB cluster.   If you don't specify a RestoreType value, then the new DB cluster is restored as a full copy of the source DB cluster. Defaults to None.
        source_db_cluster_identifier(str): The identifier of the source DB cluster from which to restore. Constraints:   Must match the identifier of an existing DBCluster.  .
        restore_to_time(str, optional): The date and time to restore the DB cluster to. Valid Values: Value must be a time in Universal Coordinated Time (UTC) format Constraints:   Must be before the latest restorable time for the DB instance   Must be specified if UseLatestRestorableTime parameter is not provided   Cannot be specified if UseLatestRestorableTime parameter is true   Cannot be specified if RestoreType parameter is copy-on-write    Example: 2015-03-07T23:45:00Z . Defaults to None.
        use_latest_restorable_time(bool, optional): A value that is set to true to restore the DB cluster to the latest restorable backup time, and false otherwise. Default: false  Constraints: Cannot be specified if RestoreToTime parameter is provided. Defaults to None.
        port(int, optional): The port number on which the new DB cluster accepts connections. Constraints: Value must be 1150-65535  Default: The same port as the original DB cluster. Defaults to None.
        db_subnet_group_name(str, optional): The DB subnet group name to use for the new DB cluster. Constraints: If supplied, must match the name of an existing DBSubnetGroup. Example: mySubnetgroup . Defaults to None.
        option_group_name(str, optional):  (Not supported by Neptune) . Defaults to None.
        vpc_security_group_ids(List, optional): A list of VPC security groups that the new DB cluster belongs to. Defaults to None.
        tags(List, optional): The tags to be applied to the restored DB cluster. Defaults to None.
        kms_key_id(str, optional): The Amazon KMS key identifier to use when restoring an encrypted DB cluster from an encrypted DB cluster. The KMS key identifier is the Amazon Resource Name (ARN) for the KMS encryption key. If you are restoring a DB cluster with the same Amazon account that owns the KMS encryption key used to encrypt the new DB cluster, then you can use the KMS key alias instead of the ARN for the KMS encryption key. You can restore to a new DB cluster and encrypt the new DB cluster with a KMS key that is different than the KMS key used to encrypt the source DB cluster. The new DB cluster is encrypted with the KMS key identified by the KmsKeyId parameter. If you do not specify a value for the KmsKeyId parameter, then the following will occur:   If the DB cluster is encrypted, then the restored DB cluster is encrypted using the KMS key that was used to encrypt the source DB cluster.   If the DB cluster is not encrypted, then the restored DB cluster is not encrypted.   If DBClusterIdentifier refers to a DB cluster that is not encrypted, then the restore request is rejected. Defaults to None.
        enable_iam_database_authentication(bool, optional): True to enable mapping of Amazon Identity and Access Management (IAM) accounts to database accounts, and otherwise false. Default: false . Defaults to None.
        enable_cloudwatch_logs_exports(List, optional): The list of logs that the restored DB cluster is to export to CloudWatch Logs. Defaults to None.
        db_cluster_parameter_group_name(str, optional): The name of the DB cluster parameter group to associate with the new DB cluster. Constraints:   If supplied, must match the name of an existing DBClusterParameterGroup.  . Defaults to None.
        deletion_protection(bool, optional): A value that indicates whether the DB cluster has deletion protection enabled. The database can't be deleted when deletion protection is enabled. By default, deletion protection is disabled. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.neptune.db.cluster.to_point_in_time.restore(
                ctx, db_cluster_identifier=value, source_db_cluster_identifier=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.neptune.db.cluster.to_point_in_time.restore db_cluster_identifier=value, source_db_cluster_identifier=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="neptune",
        operation="restore_db_cluster_to_point_in_time",
        op_kwargs={
            "DBClusterIdentifier": db_cluster_identifier,
            "RestoreType": restore_type,
            "SourceDBClusterIdentifier": source_db_cluster_identifier,
            "RestoreToTime": restore_to_time,
            "UseLatestRestorableTime": use_latest_restorable_time,
            "Port": port,
            "DBSubnetGroupName": db_subnet_group_name,
            "OptionGroupName": option_group_name,
            "VpcSecurityGroupIds": vpc_security_group_ids,
            "Tags": tags,
            "KmsKeyId": kms_key_id,
            "EnableIAMDatabaseAuthentication": enable_iam_database_authentication,
            "EnableCloudwatchLogsExports": enable_cloudwatch_logs_exports,
            "DBClusterParameterGroupName": db_cluster_parameter_group_name,
            "DeletionProtection": deletion_protection,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def modify(
    hub,
    ctx,
    replication_group_id: str,
    node_group_count: int,
    apply_immediately: bool,
    resharding_configuration: List = None,
    node_groups_to_remove: List = None,
    node_groups_to_retain: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies a replication group's shards (node groups) by allowing you to add shards, remove shards, or rebalance the keyspaces among existing shards.

    Args:
        replication_group_id(str): The name of the Redis (cluster mode enabled) cluster (replication group) on which the shards are to be configured.
        node_group_count(int): The number of node groups (shards) that results from the modification of the shard configuration.
        apply_immediately(bool): Indicates that the shard reconfiguration process begins immediately. At present, the only permitted value for this parameter is true. Value: true.
        resharding_configuration(List, optional): Specifies the preferred availability zones for each node group in the cluster. If the value of NodeGroupCount is greater than the current number of node groups (shards), you can use this parameter to specify the preferred availability zones of the cluster's shards. If you omit this parameter ElastiCache selects availability zones for you. You can specify this parameter only if the value of NodeGroupCount is greater than the current number of node groups (shards). Defaults to None.
        node_groups_to_remove(List, optional): If the value of NodeGroupCount is less than the current number of node groups (shards), then either NodeGroupsToRemove or NodeGroupsToRetain is required. NodeGroupsToRemove is a list of NodeGroupIds to remove from the cluster. ElastiCache for Redis will attempt to remove all node groups listed by NodeGroupsToRemove from the cluster. Defaults to None.
        node_groups_to_retain(List, optional): If the value of NodeGroupCount is less than the current number of node groups (shards), then either NodeGroupsToRemove or NodeGroupsToRetain is required. NodeGroupsToRetain is a list of NodeGroupIds to retain in the cluster. ElastiCache for Redis will attempt to remove all node groups except those listed by NodeGroupsToRetain from the cluster. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.replication_group.shard_configuration.modify(
                ctx, replication_group_id=value, node_group_count=value, apply_immediately=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.replication_group.shard_configuration.modify replication_group_id=value, node_group_count=value, apply_immediately=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="modify_replication_group_shard_configuration",
        op_kwargs={
            "ReplicationGroupId": replication_group_id,
            "NodeGroupCount": node_group_count,
            "ApplyImmediately": apply_immediately,
            "ReshardingConfiguration": resharding_configuration,
            "NodeGroupsToRemove": node_groups_to_remove,
            "NodeGroupsToRetain": node_groups_to_retain,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub,
    ctx,
    route_table_id: str,
    destination_cidr_block: str = None,
    destination_ipv6_cidr_block: str = None,
    destination_prefix_list_id: str = None,
    dry_run: bool = None,
    vpc_endpoint_id: str = None,
    egress_only_internet_gateway_id: str = None,
    gateway_id: str = None,
    instance_id: str = None,
    nat_gateway_id: str = None,
    transit_gateway_id: str = None,
    local_gateway_id: str = None,
    carrier_gateway_id: str = None,
    network_interface_id: str = None,
    vpc_peering_connection_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a route in a route table within a VPC. You must specify one of the following targets: internet gateway or virtual private gateway, NAT instance, NAT gateway, VPC peering connection, network interface, egress-only internet gateway, or transit gateway. When determining how to route traffic, we use the route with the most specific match. For example, traffic is destined for the IPv4 address 192.0.2.3, and the route table includes the following two IPv4 routes:    192.0.2.0/24 (goes to some target A)    192.0.2.0/28 (goes to some target B)   Both routes apply to the traffic destined for 192.0.2.3. However, the second route in the list covers a smaller number of IP addresses and is therefore more specific, so we use that route to determine where to target the traffic. For more information about route tables, see Route tables in the Amazon Virtual Private Cloud User Guide.

    Args:
        destination_cidr_block(str, optional): The IPv4 CIDR address block used for the destination match. Routing decisions are based on the most specific match. We modify the specified CIDR block to its canonical form; for example, if you specify 100.68.0.18/18, we modify it to 100.68.0.0/18. Defaults to None.
        destination_ipv6_cidr_block(str, optional): The IPv6 CIDR block used for the destination match. Routing decisions are based on the most specific match. Defaults to None.
        destination_prefix_list_id(str, optional): The ID of a prefix list used for the destination match. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        vpc_endpoint_id(str, optional): The ID of a VPC endpoint. Supported for Gateway Load Balancer endpoints only. Defaults to None.
        egress_only_internet_gateway_id(str, optional): [IPv6 traffic only] The ID of an egress-only internet gateway. Defaults to None.
        gateway_id(str, optional): The ID of an internet gateway or virtual private gateway attached to your VPC. Defaults to None.
        instance_id(str, optional): The ID of a NAT instance in your VPC. The operation fails if you specify an instance ID unless exactly one network interface is attached. Defaults to None.
        nat_gateway_id(str, optional): [IPv4 traffic only] The ID of a NAT gateway. Defaults to None.
        transit_gateway_id(str, optional): The ID of a transit gateway. Defaults to None.
        local_gateway_id(str, optional): The ID of the local gateway. Defaults to None.
        carrier_gateway_id(str, optional): The ID of the carrier gateway. You can only use this option when the VPC contains a subnet which is associated with a Wavelength Zone. Defaults to None.
        network_interface_id(str, optional): The ID of a network interface. Defaults to None.
        route_table_id(str): The ID of the route table for the route.
        vpc_peering_connection_id(str, optional): The ID of a VPC peering connection. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.route.init.create(ctx, route_table_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.route.init.create route_table_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="create_route",
        op_kwargs={
            "DestinationCidrBlock": destination_cidr_block,
            "DestinationIpv6CidrBlock": destination_ipv6_cidr_block,
            "DestinationPrefixListId": destination_prefix_list_id,
            "DryRun": dry_run,
            "VpcEndpointId": vpc_endpoint_id,
            "EgressOnlyInternetGatewayId": egress_only_internet_gateway_id,
            "GatewayId": gateway_id,
            "InstanceId": instance_id,
            "NatGatewayId": nat_gateway_id,
            "TransitGatewayId": transit_gateway_id,
            "LocalGatewayId": local_gateway_id,
            "CarrierGatewayId": carrier_gateway_id,
            "NetworkInterfaceId": network_interface_id,
            "RouteTableId": route_table_id,
            "VpcPeeringConnectionId": vpc_peering_connection_id,
        },
    )


async def delete(
    hub,
    ctx,
    route_table_id: str,
    destination_cidr_block: str = None,
    destination_ipv6_cidr_block: str = None,
    destination_prefix_list_id: str = None,
    dry_run: bool = None,
) -> None:
    r"""
    **Autogenerated function**

    Deletes the specified route from the specified route table.

    Args:
        destination_cidr_block(str, optional): The IPv4 CIDR range for the route. The value you specify must match the CIDR for the route exactly. Defaults to None.
        destination_ipv6_cidr_block(str, optional): The IPv6 CIDR range for the route. The value you specify must match the CIDR for the route exactly. Defaults to None.
        destination_prefix_list_id(str, optional): The ID of the prefix list for the route. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        route_table_id(str): The ID of the route table.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.route.init.delete(ctx, route_table_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.route.init.delete route_table_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="delete_route",
        op_kwargs={
            "DestinationCidrBlock": destination_cidr_block,
            "DestinationIpv6CidrBlock": destination_ipv6_cidr_block,
            "DestinationPrefixListId": destination_prefix_list_id,
            "DryRun": dry_run,
            "RouteTableId": route_table_id,
        },
    )


async def replace(
    hub,
    ctx,
    route_table_id: str,
    destination_cidr_block: str = None,
    destination_ipv6_cidr_block: str = None,
    destination_prefix_list_id: str = None,
    dry_run: bool = None,
    vpc_endpoint_id: str = None,
    egress_only_internet_gateway_id: str = None,
    gateway_id: str = None,
    instance_id: str = None,
    local_target: bool = None,
    nat_gateway_id: str = None,
    transit_gateway_id: str = None,
    local_gateway_id: str = None,
    carrier_gateway_id: str = None,
    network_interface_id: str = None,
    vpc_peering_connection_id: str = None,
) -> None:
    r"""
    **Autogenerated function**

    Replaces an existing route within a route table in a VPC. You must provide only one of the following: internet gateway, virtual private gateway, NAT instance, NAT gateway, VPC peering connection, network interface, egress-only internet gateway, or transit gateway. For more information, see Route tables in the Amazon Virtual Private Cloud User Guide.

    Args:
        destination_cidr_block(str, optional): The IPv4 CIDR address block used for the destination match. The value that you provide must match the CIDR of an existing route in the table. Defaults to None.
        destination_ipv6_cidr_block(str, optional): The IPv6 CIDR address block used for the destination match. The value that you provide must match the CIDR of an existing route in the table. Defaults to None.
        destination_prefix_list_id(str, optional): The ID of the prefix list for the route. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        vpc_endpoint_id(str, optional): The ID of a VPC endpoint. Supported for Gateway Load Balancer endpoints only. Defaults to None.
        egress_only_internet_gateway_id(str, optional): [IPv6 traffic only] The ID of an egress-only internet gateway. Defaults to None.
        gateway_id(str, optional): The ID of an internet gateway or virtual private gateway. Defaults to None.
        instance_id(str, optional): The ID of a NAT instance in your VPC. Defaults to None.
        local_target(bool, optional): Specifies whether to reset the local route to its default target (local). Defaults to None.
        nat_gateway_id(str, optional): [IPv4 traffic only] The ID of a NAT gateway. Defaults to None.
        transit_gateway_id(str, optional): The ID of a transit gateway. Defaults to None.
        local_gateway_id(str, optional): The ID of the local gateway. Defaults to None.
        carrier_gateway_id(str, optional): [IPv4 traffic only] The ID of a carrier gateway. Defaults to None.
        network_interface_id(str, optional): The ID of a network interface. Defaults to None.
        route_table_id(str): The ID of the route table.
        vpc_peering_connection_id(str, optional): The ID of a VPC peering connection. Defaults to None.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.route.init.replace(ctx, route_table_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.route.init.replace route_table_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="replace_route",
        op_kwargs={
            "DestinationCidrBlock": destination_cidr_block,
            "DestinationIpv6CidrBlock": destination_ipv6_cidr_block,
            "DestinationPrefixListId": destination_prefix_list_id,
            "DryRun": dry_run,
            "VpcEndpointId": vpc_endpoint_id,
            "EgressOnlyInternetGatewayId": egress_only_internet_gateway_id,
            "GatewayId": gateway_id,
            "InstanceId": instance_id,
            "LocalTarget": local_target,
            "NatGatewayId": nat_gateway_id,
            "TransitGatewayId": transit_gateway_id,
            "LocalGatewayId": local_gateway_id,
            "CarrierGatewayId": carrier_gateway_id,
            "NetworkInterfaceId": network_interface_id,
            "RouteTableId": route_table_id,
            "VpcPeeringConnectionId": vpc_peering_connection_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def update(hub, ctx, name: str, permissions_mode: str) -> Dict:
    r"""
    **Autogenerated function**

    Updates the permissions mode of a ledger.  Before you switch to the STANDARD permissions mode, you must first create all required IAM policies and table tags to avoid disruption to your users. To learn more, see Migrating to the standard permissions mode in the Amazon QLDB Developer Guide.

    Args:
        name(str): The name of the ledger.
        permissions_mode(str): The permissions mode to assign to the ledger. This parameter can have one of the following values:    ALLOW_ALL: A legacy permissions mode that enables access control with API-level granularity for ledgers. This mode allows users who have the SendCommand API permission for this ledger to run all PartiQL commands (hence, ALLOW_ALL) on any tables in the specified ledger. This mode disregards any table-level or command-level IAM permissions policies that you create for the ledger.    STANDARD: (Recommended) A permissions mode that enables access control with finer granularity for ledgers, tables, and PartiQL commands. By default, this mode denies all user requests to run any PartiQL commands on any tables in this ledger. To allow PartiQL commands to run, you must create IAM permissions policies for specific table resources and PartiQL actions, in addition to the SendCommand API permission for the ledger. For information, see Getting started with the standard permissions mode in the Amazon QLDB Developer Guide.    We strongly recommend using the STANDARD permissions mode to maximize the security of your ledger data. .

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.qldb.ledger.permission_mode.update(
                ctx, name=value, permissions_mode=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.qldb.ledger.permission_mode.update name=value, permissions_mode=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="qldb",
        operation="update_ledger_permissions_mode",
        op_kwargs={"Name": name, "PermissionsMode": permissions_mode},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def modify(
    hub,
    ctx,
    vpn_connection_id: str,
    vpn_tunnel_outside_ip_address: str,
    tunnel_options: Dict,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies the options for a VPN tunnel in an AWS Site-to-Site VPN connection. You can modify multiple options for a tunnel in a single request, but you can only modify one tunnel at a time. For more information, see Site-to-Site VPN Tunnel Options for Your Site-to-Site VPN Connection in the AWS Site-to-Site VPN User Guide.

    Args:
        vpn_connection_id(str): The ID of the AWS Site-to-Site VPN connection.
        vpn_tunnel_outside_ip_address(str): The external IP address of the VPN tunnel.
        tunnel_options(Dict): The tunnel options to modify.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.vpn.tunnel.option.modify(
                ctx,
                vpn_connection_id=value,
                vpn_tunnel_outside_ip_address=value,
                tunnel_options=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.vpn.tunnel.option.modify vpn_connection_id=value, vpn_tunnel_outside_ip_address=value, tunnel_options=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="modify_vpn_tunnel_options",
        op_kwargs={
            "VpnConnectionId": vpn_connection_id,
            "VpnTunnelOutsideIpAddress": vpn_tunnel_outside_ip_address,
            "TunnelOptions": tunnel_options,
            "DryRun": dry_run,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def describe_all(
    hub,
    ctx,
    directory_id: str,
    domain_controller_ids: List = None,
    next_token: str = None,
    limit: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Provides information about any domain controllers in your directory.

    Args:
        directory_id(str): Identifier of the directory for which to retrieve the domain controller information.
        domain_controller_ids(List, optional): A list of identifiers for the domain controllers whose information will be provided. Defaults to None.
        next_token(str, optional): The DescribeDomainControllers.NextToken value from a previous call to DescribeDomainControllers. Pass null if this is the first call. . Defaults to None.
        limit(int, optional): The maximum number of items to return. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ds.domain_controller.describe_all(ctx, directory_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ds.domain_controller.describe_all directory_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ds",
        operation="describe_domain_controllers",
        op_kwargs={
            "DirectoryId": directory_id,
            "DomainControllerIds": domain_controller_ids,
            "NextToken": next_token,
            "Limit": limit,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe_all(hub, ctx) -> Dict:
    r"""
    **Autogenerated function**

    Describes event configurations.


    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.event_configuration.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.event_configuration.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx, service_name="iot", operation="describe_event_configurations", op_kwargs={}
    )


async def update_multiple(hub, ctx, event_configurations: Dict = None) -> Dict:
    r"""
    **Autogenerated function**

    Updates the event configurations.

    Args:
        event_configurations(Dict, optional): The new event configuration values. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot.event_configuration.update_multiple(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot.event_configuration.update_multiple
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot",
        operation="update_event_configurations",
        op_kwargs={"eventConfigurations": event_configurations},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def recognize(
    hub,
    ctx,
    bot_id: str,
    bot_alias_id: str,
    locale_id: str,
    session_id: str,
    request_content_type: str,
    session_state: str = None,
    request_attributes: str = None,
    response_content_type: str = None,
    input_stream: bytes = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Sends user input to Amazon Lex V2. You can send text or speech. Clients use this API to send text and audio requests to Amazon Lex V2 at runtime. Amazon Lex V2 interprets the user input using the machine learning model built for the bot. The following request fields must be compressed with gzip and then base64 encoded before you send them to Amazon Lex V2.    requestAttributes   sessionState   The following response fields are compressed using gzip and then base64 encoded by Amazon Lex V2. Before you can use these fields, you must decode and decompress them.    inputTranscript   interpretations   messages   requestAttributes   sessionState   The example contains a Java application that compresses and encodes a Java object to send to Amazon Lex V2, and a second that decodes and decompresses a response from Amazon Lex V2.

    Args:
        bot_id(str): The identifier of the bot that should receive the request.
        bot_alias_id(str): The alias identifier in use for the bot that should receive the request.
        locale_id(str): The locale where the session is in use.
        session_id(str): The identifier of the session in use.
        session_state(str, optional): Sets the state of the session with the user. You can use this to set the current intent, attributes, context, and dialog action. Use the dialog action to determine the next step that Amazon Lex V2 should use in the conversation with the user. The sessionState field must be compressed using gzip and then base64 encoded before sending to Amazon Lex V2. Defaults to None.
        request_attributes(str, optional): Request-specific information passed between the client application and Amazon Lex V2  The namespace x-amz-lex: is reserved for special attributes. Don't create any request attributes for prefix x-amz-lex:. The requestAttributes field must be compressed using gzip and then base64 encoded before sending to Amazon Lex V2. Defaults to None.
        request_content_type(str): Indicates the format for audio input or that the content is text. The header must start with one of the following prefixes:   PCM format, audio data must be in little-endian byte order.   audio/l16; rate=16000; channels=1   audio/x-l16; sample-rate=16000; channel-count=1   audio/lpcm; sample-rate=8000; sample-size-bits=16; channel-count=1; is-big-endian=false     Opus format   audio/x-cbr-opus-with-preamble;preamble-size=0;bit-rate=256000;frame-size-milliseconds=4     Text format   text/plain; charset=utf-8    .
        response_content_type(str, optional): The message that Amazon Lex V2 returns in the response can be either text or speech based on the responseContentType value.   If the value is text/plain;charset=utf-8, Amazon Lex V2 returns text in the response.   If the value begins with audio/, Amazon Lex V2 returns speech in the response. Amazon Lex V2 uses Amazon Polly to generate the speech using the configuration that you specified in the requestContentType parameter. For example, if you specify audio/mpeg as the value, Amazon Lex V2 returns speech in the MPEG format.   If the value is audio/pcm, the speech returned is audio/pcm at 16 KHz in 16-bit, little-endian format.   The following are the accepted values:   audio/mpeg   audio/ogg   audio/pcm (16 KHz)   audio/* (defaults to mpeg)   text/plain; charset=utf-8    . Defaults to None.
        input_stream(bytes, optional): User input in PCM or Opus audio format or text format as described in the requestContentType parameter. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lexv2_runtime.utterance.recognize(
                ctx,
                bot_id=value,
                bot_alias_id=value,
                locale_id=value,
                session_id=value,
                request_content_type=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lexv2_runtime.utterance.recognize bot_id=value, bot_alias_id=value, locale_id=value, session_id=value, request_content_type=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lexv2-runtime",
        operation="recognize_utterance",
        op_kwargs={
            "botId": bot_id,
            "botAliasId": bot_alias_id,
            "localeId": locale_id,
            "sessionId": session_id,
            "sessionState": session_state,
            "requestAttributes": request_attributes,
            "requestContentType": request_content_type,
            "responseContentType": response_content_type,
            "inputStream": input_stream,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    replication_group_id: str,
    replication_group_description: str,
    global_replication_group_id: str = None,
    primary_cluster_id: str = None,
    automatic_failover_enabled: bool = None,
    multi_az_enabled: bool = None,
    num_cache_clusters: int = None,
    preferred_cache_cluster_a_zs: List = None,
    num_node_groups: int = None,
    replicas_per_node_group: int = None,
    node_group_configuration: List = None,
    cache_node_type: str = None,
    engine: str = None,
    engine_version: str = None,
    cache_parameter_group_name: str = None,
    cache_subnet_group_name: str = None,
    cache_security_group_names: List = None,
    security_group_ids: List = None,
    tags: List = None,
    snapshot_arns: List = None,
    snapshot_name: str = None,
    preferred_maintenance_window: str = None,
    port: int = None,
    notification_topic_arn: str = None,
    auto_minor_version_upgrade: bool = None,
    snapshot_retention_limit: int = None,
    snapshot_window: str = None,
    auth_token: str = None,
    transit_encryption_enabled: bool = None,
    at_rest_encryption_enabled: bool = None,
    kms_key_id: str = None,
    user_group_ids: List = None,
    log_delivery_configurations: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a Redis (cluster mode disabled) or a Redis (cluster mode enabled) replication group. This API can be used to create a standalone regional replication group or a secondary replication group associated with a Global datastore. A Redis (cluster mode disabled) replication group is a collection of clusters, where one of the clusters is a read/write primary and the others are read-only replicas. Writes to the primary are asynchronously propagated to the replicas. A Redis cluster-mode enabled cluster is comprised of from 1 to 90 shards (API/CLI: node groups). Each shard has a primary node and up to 5 read-only replica nodes. The configuration can range from 90 shards and 0 replicas to 15 shards and 5 replicas, which is the maximum number or replicas allowed.  The node or shard limit can be increased to a maximum of 500 per cluster if the Redis engine version is 5.0.6 or higher. For example, you can choose to configure a 500 node cluster that ranges between 83 shards (one primary and 5 replicas per shard) and 500 shards (single primary and no replicas). Make sure there are enough available IP addresses to accommodate the increase. Common pitfalls include the subnets in the subnet group have too small a CIDR range or the subnets are shared and heavily used by other clusters. For more information, see Creating a Subnet Group. For versions below 5.0.6, the limit is 250 per cluster. To request a limit increase, see AWS Service Limits and choose the limit type Nodes per cluster per instance type.  When a Redis (cluster mode disabled) replication group has been successfully created, you can add one or more read replicas to it, up to a total of 5 read replicas. If you need to increase or decrease the number of node groups (console: shards), you can avail yourself of ElastiCache for Redis' scaling. For more information, see Scaling ElastiCache for Redis Clusters in the ElastiCache User Guide.  This operation is valid for Redis only.

    Args:
        replication_group_id(str): The replication group identifier. This parameter is stored as a lowercase string. Constraints:   A name must contain from 1 to 40 alphanumeric characters or hyphens.   The first character must be a letter.   A name cannot end with a hyphen or contain two consecutive hyphens.  .
        replication_group_description(str): A user-created description for the replication group.
        global_replication_group_id(str, optional): The name of the Global datastore. Defaults to None.
        primary_cluster_id(str, optional): The identifier of the cluster that serves as the primary for this replication group. This cluster must already exist and have a status of available. This parameter is not required if NumCacheClusters, NumNodeGroups, or ReplicasPerNodeGroup is specified. Defaults to None.
        automatic_failover_enabled(bool, optional): Specifies whether a read-only replica is automatically promoted to read/write primary if the existing primary fails.  AutomaticFailoverEnabled must be enabled for Redis (cluster mode enabled) replication groups. Default: false. Defaults to None.
        multi_az_enabled(bool, optional): A flag indicating if you have Multi-AZ enabled to enhance fault tolerance. For more information, see Minimizing Downtime: Multi-AZ. Defaults to None.
        num_cache_clusters(int, optional): The number of clusters this replication group initially has. This parameter is not used if there is more than one node group (shard). You should use ReplicasPerNodeGroup instead. If AutomaticFailoverEnabled is true, the value of this parameter must be at least 2. If AutomaticFailoverEnabled is false you can omit this parameter (it will default to 1), or you can explicitly set it to a value between 2 and 6. The maximum permitted value for NumCacheClusters is 6 (1 primary plus 5 replicas). Defaults to None.
        preferred_cache_cluster_a_zs(List, optional): A list of EC2 Availability Zones in which the replication group's clusters are created. The order of the Availability Zones in the list is the order in which clusters are allocated. The primary cluster is created in the first AZ in the list. This parameter is not used if there is more than one node group (shard). You should use NodeGroupConfiguration instead.  If you are creating your replication group in an Amazon VPC (recommended), you can only locate clusters in Availability Zones associated with the subnets in the selected subnet group. The number of Availability Zones listed must equal the value of NumCacheClusters.  Default: system chosen Availability Zones. Defaults to None.
        num_node_groups(int, optional): An optional parameter that specifies the number of node groups (shards) for this Redis (cluster mode enabled) replication group. For Redis (cluster mode disabled) either omit this parameter or set it to 1. Default: 1. Defaults to None.
        replicas_per_node_group(int, optional): An optional parameter that specifies the number of replica nodes in each node group (shard). Valid values are 0 to 5. Defaults to None.
        node_group_configuration(List, optional): A list of node group (shard) configuration options. Each node group (shard) configuration has the following members: PrimaryAvailabilityZone, ReplicaAvailabilityZones, ReplicaCount, and Slots. If you're creating a Redis (cluster mode disabled) or a Redis (cluster mode enabled) replication group, you can use this parameter to individually configure each node group (shard), or you can omit this parameter. However, it is required when seeding a Redis (cluster mode enabled) cluster from a S3 rdb file. You must configure each node group (shard) using this parameter because you must specify the slots for each node group. Defaults to None.
        cache_node_type(str, optional): The compute and memory capacity of the nodes in the node group (shard). The following node types are supported by ElastiCache. Generally speaking, the current generation types provide more memory and computational power at lower cost when compared to their equivalent previous generation counterparts.   General purpose:   Current generation:   M6g node types (available only for Redis engine version 5.0.6 onward and for Memcached engine version 1.5.16 onward).  cache.m6g.large, cache.m6g.xlarge, cache.m6g.2xlarge, cache.m6g.4xlarge, cache.m6g.8xlarge, cache.m6g.12xlarge, cache.m6g.16xlarge   For region availability, see Supported Node Types    M5 node types: cache.m5.large, cache.m5.xlarge, cache.m5.2xlarge, cache.m5.4xlarge, cache.m5.12xlarge, cache.m5.24xlarge   M4 node types: cache.m4.large, cache.m4.xlarge, cache.m4.2xlarge, cache.m4.4xlarge, cache.m4.10xlarge   T3 node types: cache.t3.micro, cache.t3.small, cache.t3.medium   T2 node types: cache.t2.micro, cache.t2.small, cache.t2.medium    Previous generation: (not recommended)  T1 node types: cache.t1.micro   M1 node types: cache.m1.small, cache.m1.medium, cache.m1.large, cache.m1.xlarge   M3 node types: cache.m3.medium, cache.m3.large, cache.m3.xlarge, cache.m3.2xlarge      Compute optimized:   Previous generation: (not recommended)  C1 node types: cache.c1.xlarge      Memory optimized:   Current generation:   R6g node types (available only for Redis engine version 5.0.6 onward and for Memcached engine version 1.5.16 onward).  cache.r6g.large, cache.r6g.xlarge, cache.r6g.2xlarge, cache.r6g.4xlarge, cache.r6g.8xlarge, cache.r6g.12xlarge, cache.r6g.16xlarge   For region availability, see Supported Node Types    R5 node types: cache.r5.large, cache.r5.xlarge, cache.r5.2xlarge, cache.r5.4xlarge, cache.r5.12xlarge, cache.r5.24xlarge   R4 node types: cache.r4.large, cache.r4.xlarge, cache.r4.2xlarge, cache.r4.4xlarge, cache.r4.8xlarge, cache.r4.16xlarge    Previous generation: (not recommended)  M2 node types: cache.m2.xlarge, cache.m2.2xlarge, cache.m2.4xlarge   R3 node types: cache.r3.large, cache.r3.xlarge, cache.r3.2xlarge, cache.r3.4xlarge, cache.r3.8xlarge       Additional node type info    All current generation instance types are created in Amazon VPC by default.   Redis append-only files (AOF) are not supported for T1 or T2 instances.   Redis Multi-AZ with automatic failover is not supported on T1 instances.   Redis configuration variables appendonly and appendfsync are not supported on Redis version 2.8.22 and later.  . Defaults to None.
        engine(str, optional): The name of the cache engine to be used for the clusters in this replication group. Must be Redis. Defaults to None.
        engine_version(str, optional): The version number of the cache engine to be used for the clusters in this replication group. To view the supported cache engine versions, use the DescribeCacheEngineVersions operation.  Important: You can upgrade to a newer engine version (see Selecting a Cache Engine and Version) in the ElastiCache User Guide, but you cannot downgrade to an earlier engine version. If you want to use an earlier engine version, you must delete the existing cluster or replication group and create it anew with the earlier engine version. . Defaults to None.
        cache_parameter_group_name(str, optional): The name of the parameter group to associate with this replication group. If this argument is omitted, the default cache parameter group for the specified engine is used. If you are running Redis version 3.2.4 or later, only one node group (shard), and want to use a default parameter group, we recommend that you specify the parameter group by name.    To create a Redis (cluster mode disabled) replication group, use CacheParameterGroupName=default.redis3.2.   To create a Redis (cluster mode enabled) replication group, use CacheParameterGroupName=default.redis3.2.cluster.on.  . Defaults to None.
        cache_subnet_group_name(str, optional): The name of the cache subnet group to be used for the replication group.  If you're going to launch your cluster in an Amazon VPC, you need to create a subnet group before you start creating a cluster. For more information, see Subnets and Subnet Groups. . Defaults to None.
        cache_security_group_names(List, optional): A list of cache security group names to associate with this replication group. Defaults to None.
        security_group_ids(List, optional): One or more Amazon VPC security groups associated with this replication group. Use this parameter only when you are creating a replication group in an Amazon Virtual Private Cloud (Amazon VPC). Defaults to None.
        tags(List, optional): A list of tags to be added to this resource. Tags are comma-separated key,value pairs (e.g. Key=myKey, Value=myKeyValue. You can include multiple tags as shown following: Key=myKey, Value=myKeyValue Key=mySecondKey, Value=mySecondKeyValue. Tags on replication groups will be replicated to all nodes. Defaults to None.
        snapshot_arns(List, optional): A list of Amazon Resource Names (ARN) that uniquely identify the Redis RDB snapshot files stored in Amazon S3. The snapshot files are used to populate the new replication group. The Amazon S3 object name in the ARN cannot contain any commas. The new replication group will have the number of node groups (console: shards) specified by the parameter NumNodeGroups or the number of node groups configured by NodeGroupConfiguration regardless of the number of ARNs specified here. Example of an Amazon S3 ARN: arn:aws:s3:::my_bucket/snapshot1.rdb . Defaults to None.
        snapshot_name(str, optional): The name of a snapshot from which to restore data into the new replication group. The snapshot status changes to restoring while the new replication group is being created. Defaults to None.
        preferred_maintenance_window(str, optional): Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for ddd are: Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for ddd are:    sun     mon     tue     wed     thu     fri     sat    Example: sun:23:00-mon:01:30 . Defaults to None.
        port(int, optional): The port number on which each member of the replication group accepts connections. Defaults to None.
        notification_topic_arn(str, optional): The Amazon Resource Name (ARN) of the Amazon Simple Notification Service (SNS) topic to which notifications are sent.  The Amazon SNS topic owner must be the same as the cluster owner. . Defaults to None.
        auto_minor_version_upgrade(bool, optional): This parameter is currently disabled. Defaults to None.
        snapshot_retention_limit(int, optional): The number of days for which ElastiCache retains automatic snapshots before deleting them. For example, if you set SnapshotRetentionLimit to 5, a snapshot that was taken today is retained for 5 days before being deleted. Default: 0 (i.e., automatic backups are disabled for this cluster). Defaults to None.
        snapshot_window(str, optional): The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of your node group (shard). Example: 05:00-09:00  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range. Defaults to None.
        auth_token(str, optional):  Reserved parameter. The password used to access a password protected server.  AuthToken can be specified only on replication groups where TransitEncryptionEnabled is true.  For HIPAA compliance, you must specify TransitEncryptionEnabled as true, an AuthToken, and a CacheSubnetGroup.  Password constraints:   Must be only printable ASCII characters.   Must be at least 16 characters and no more than 128 characters in length.   The only permitted printable special characters are !, &, #, $, ^, <, >, and -. Other printable special characters cannot be used in the AUTH token.   For more information, see AUTH password at http://redis.io/commands/AUTH. Defaults to None.
        transit_encryption_enabled(bool, optional): A flag that enables in-transit encryption when set to true. You cannot modify the value of TransitEncryptionEnabled after the cluster is created. To enable in-transit encryption on a cluster you must set TransitEncryptionEnabled to true when you create a cluster. This parameter is valid only if the Engine parameter is redis, the EngineVersion parameter is 3.2.6, 4.x or later, and the cluster is being created in an Amazon VPC. If you enable in-transit encryption, you must also specify a value for CacheSubnetGroup.  Required: Only available when creating a replication group in an Amazon VPC using redis version 3.2.6, 4.x or later. Default: false   For HIPAA compliance, you must specify TransitEncryptionEnabled as true, an AuthToken, and a CacheSubnetGroup. . Defaults to None.
        at_rest_encryption_enabled(bool, optional): A flag that enables encryption at rest when set to true. You cannot modify the value of AtRestEncryptionEnabled after the replication group is created. To enable encryption at rest on a replication group you must set AtRestEncryptionEnabled to true when you create the replication group.   Required: Only available when creating a replication group in an Amazon VPC using redis version 3.2.6, 4.x or later. Default: false . Defaults to None.
        kms_key_id(str, optional): The ID of the KMS key used to encrypt the disk in the cluster. Defaults to None.
        user_group_ids(List, optional): The user group to associate with the replication group. Defaults to None.
        log_delivery_configurations(List, optional): Specifies the destination, format and type of the logs. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.replication_group.init.create(
                ctx, replication_group_id=value, replication_group_description=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.replication_group.init.create replication_group_id=value, replication_group_description=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="create_replication_group",
        op_kwargs={
            "ReplicationGroupId": replication_group_id,
            "ReplicationGroupDescription": replication_group_description,
            "GlobalReplicationGroupId": global_replication_group_id,
            "PrimaryClusterId": primary_cluster_id,
            "AutomaticFailoverEnabled": automatic_failover_enabled,
            "MultiAZEnabled": multi_az_enabled,
            "NumCacheClusters": num_cache_clusters,
            "PreferredCacheClusterAZs": preferred_cache_cluster_a_zs,
            "NumNodeGroups": num_node_groups,
            "ReplicasPerNodeGroup": replicas_per_node_group,
            "NodeGroupConfiguration": node_group_configuration,
            "CacheNodeType": cache_node_type,
            "Engine": engine,
            "EngineVersion": engine_version,
            "CacheParameterGroupName": cache_parameter_group_name,
            "CacheSubnetGroupName": cache_subnet_group_name,
            "CacheSecurityGroupNames": cache_security_group_names,
            "SecurityGroupIds": security_group_ids,
            "Tags": tags,
            "SnapshotArns": snapshot_arns,
            "SnapshotName": snapshot_name,
            "PreferredMaintenanceWindow": preferred_maintenance_window,
            "Port": port,
            "NotificationTopicArn": notification_topic_arn,
            "AutoMinorVersionUpgrade": auto_minor_version_upgrade,
            "SnapshotRetentionLimit": snapshot_retention_limit,
            "SnapshotWindow": snapshot_window,
            "AuthToken": auth_token,
            "TransitEncryptionEnabled": transit_encryption_enabled,
            "AtRestEncryptionEnabled": at_rest_encryption_enabled,
            "KmsKeyId": kms_key_id,
            "UserGroupIds": user_group_ids,
            "LogDeliveryConfigurations": log_delivery_configurations,
        },
    )


async def delete(
    hub,
    ctx,
    replication_group_id: str,
    retain_primary_cluster: bool = None,
    final_snapshot_identifier: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Deletes an existing replication group. By default, this operation deletes the entire replication group, including the primary/primaries and all of the read replicas. If the replication group has only one primary, you can optionally delete only the read replicas, while retaining the primary by setting RetainPrimaryCluster=true. When you receive a successful response from this operation, Amazon ElastiCache immediately begins deleting the selected resources; you cannot cancel or revert this operation.  This operation is valid for Redis only.

    Args:
        replication_group_id(str): The identifier for the cluster to be deleted. This parameter is not case sensitive.
        retain_primary_cluster(bool, optional): If set to true, all of the read replicas are deleted, but the primary node is retained. Defaults to None.
        final_snapshot_identifier(str, optional): The name of a final node group (shard) snapshot. ElastiCache creates the snapshot from the primary node in the cluster, rather than one of the replicas; this is to ensure that it captures the freshest data. After the final snapshot is taken, the replication group is immediately deleted. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.replication_group.init.delete(
                ctx, replication_group_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.replication_group.init.delete replication_group_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="delete_replication_group",
        op_kwargs={
            "ReplicationGroupId": replication_group_id,
            "RetainPrimaryCluster": retain_primary_cluster,
            "FinalSnapshotIdentifier": final_snapshot_identifier,
        },
    )


async def describe_all(
    hub,
    ctx,
    replication_group_id: str = None,
    max_records: int = None,
    marker: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns information about a particular replication group. If no identifier is specified, DescribeReplicationGroups returns information about all replication groups.  This operation is valid for Redis only.

    Args:
        replication_group_id(str, optional): The identifier for the replication group to be described. This parameter is not case sensitive. If you do not specify this parameter, information about all replication groups is returned. Defaults to None.
        max_records(int, optional): The maximum number of records to include in the response. If more records exist than the specified MaxRecords value, a marker is included in the response so that the remaining results can be retrieved. Default: 100 Constraints: minimum 20; maximum 100. Defaults to None.
        marker(str, optional): An optional marker returned from a prior request. Use this marker for pagination of results from this operation. If this parameter is specified, the response includes only records beyond the marker, up to the value specified by MaxRecords. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.replication_group.init.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.replication_group.init.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="describe_replication_groups",
        op_kwargs={
            "ReplicationGroupId": replication_group_id,
            "MaxRecords": max_records,
            "Marker": marker,
        },
    )


async def modify(
    hub,
    ctx,
    replication_group_id: str,
    replication_group_description: str = None,
    primary_cluster_id: str = None,
    snapshotting_cluster_id: str = None,
    automatic_failover_enabled: bool = None,
    multi_az_enabled: bool = None,
    node_group_id: str = None,
    cache_security_group_names: List = None,
    security_group_ids: List = None,
    preferred_maintenance_window: str = None,
    notification_topic_arn: str = None,
    cache_parameter_group_name: str = None,
    notification_topic_status: str = None,
    apply_immediately: bool = None,
    engine_version: str = None,
    auto_minor_version_upgrade: bool = None,
    snapshot_retention_limit: int = None,
    snapshot_window: str = None,
    cache_node_type: str = None,
    auth_token: str = None,
    auth_token_update_strategy: str = None,
    user_group_ids_to_add: List = None,
    user_group_ids_to_remove: List = None,
    remove_user_groups: bool = None,
    log_delivery_configurations: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies the settings for a replication group.    Scaling for Amazon ElastiCache for Redis (cluster mode enabled) in the ElastiCache User Guide    ModifyReplicationGroupShardConfiguration in the ElastiCache API Reference    This operation is valid for Redis only.

    Args:
        replication_group_id(str): The identifier of the replication group to modify.
        replication_group_description(str, optional): A description for the replication group. Maximum length is 255 characters. Defaults to None.
        primary_cluster_id(str, optional): For replication groups with a single primary, if this parameter is specified, ElastiCache promotes the specified cluster in the specified replication group to the primary role. The nodes of all other clusters in the replication group are read replicas. Defaults to None.
        snapshotting_cluster_id(str, optional): The cluster ID that is used as the daily snapshot source for the replication group. This parameter cannot be set for Redis (cluster mode enabled) replication groups. Defaults to None.
        automatic_failover_enabled(bool, optional): Determines whether a read replica is automatically promoted to read/write primary if the existing primary encounters a failure. Valid values: true | false . Defaults to None.
        multi_az_enabled(bool, optional): A list of tags to be added to this resource. A tag is a key-value pair. A tag key must be accompanied by a tag value, although null is accepted. Defaults to None.
        node_group_id(str, optional): Deprecated. This parameter is not used. Defaults to None.
        cache_security_group_names(List, optional): A list of cache security group names to authorize for the clusters in this replication group. This change is asynchronously applied as soon as possible. This parameter can be used only with replication group containing clusters running outside of an Amazon Virtual Private Cloud (Amazon VPC). Constraints: Must contain no more than 255 alphanumeric characters. Must not be Default. Defaults to None.
        security_group_ids(List, optional): Specifies the VPC Security Groups associated with the clusters in the replication group. This parameter can be used only with replication group containing clusters running in an Amazon Virtual Private Cloud (Amazon VPC). Defaults to None.
        preferred_maintenance_window(str, optional): Specifies the weekly time range during which maintenance on the cluster is performed. It is specified as a range in the format ddd:hh24:mi-ddd:hh24:mi (24H Clock UTC). The minimum maintenance window is a 60 minute period. Valid values for ddd are:    sun     mon     tue     wed     thu     fri     sat    Example: sun:23:00-mon:01:30 . Defaults to None.
        notification_topic_arn(str, optional): The Amazon Resource Name (ARN) of the Amazon SNS topic to which notifications are sent.  The Amazon SNS topic owner must be same as the replication group owner.  . Defaults to None.
        cache_parameter_group_name(str, optional): The name of the cache parameter group to apply to all of the clusters in this replication group. This change is asynchronously applied as soon as possible for parameters when the ApplyImmediately parameter is specified as true for this request. Defaults to None.
        notification_topic_status(str, optional): The status of the Amazon SNS notification topic for the replication group. Notifications are sent only if the status is active. Valid values: active | inactive . Defaults to None.
        apply_immediately(bool, optional): If true, this parameter causes the modifications in this request and any pending modifications to be applied, asynchronously and as soon as possible, regardless of the PreferredMaintenanceWindow setting for the replication group. If false, changes to the nodes in the replication group are applied on the next maintenance reboot, or the next failure reboot, whichever occurs first. Valid values: true | false  Default: false . Defaults to None.
        engine_version(str, optional): The upgraded version of the cache engine to be run on the clusters in the replication group.  Important: You can upgrade to a newer engine version (see Selecting a Cache Engine and Version), but you cannot downgrade to an earlier engine version. If you want to use an earlier engine version, you must delete the existing replication group and create it anew with the earlier engine version. . Defaults to None.
        auto_minor_version_upgrade(bool, optional): This parameter is currently disabled. Defaults to None.
        snapshot_retention_limit(int, optional): The number of days for which ElastiCache retains automatic node group (shard) snapshots before deleting them. For example, if you set SnapshotRetentionLimit to 5, a snapshot that was taken today is retained for 5 days before being deleted.  Important If the value of SnapshotRetentionLimit is set to zero (0), backups are turned off. Defaults to None.
        snapshot_window(str, optional): The daily time range (in UTC) during which ElastiCache begins taking a daily snapshot of the node group (shard) specified by SnapshottingClusterId. Example: 05:00-09:00  If you do not specify this parameter, ElastiCache automatically chooses an appropriate time range. Defaults to None.
        cache_node_type(str, optional): A valid cache node type that you want to scale this replication group to. Defaults to None.
        auth_token(str, optional): Reserved parameter. The password used to access a password protected server. This parameter must be specified with the auth-token-update-strategy  parameter. Password constraints:   Must be only printable ASCII characters   Must be at least 16 characters and no more than 128 characters in length   Cannot contain any of the following characters: '/', '"', or '@', '%'    For more information, see AUTH password at AUTH. Defaults to None.
        auth_token_update_strategy(str, optional): Specifies the strategy to use to update the AUTH token. This parameter must be specified with the auth-token parameter. Possible values:   Rotate   Set    For more information, see Authenticating Users with Redis AUTH . Defaults to None.
        user_group_ids_to_add(List, optional): The user group you are associating with the replication group. Defaults to None.
        user_group_ids_to_remove(List, optional): The user group to remove, meaning the users in the group no longer can access the replication group. Defaults to None.
        remove_user_groups(bool, optional): Removes the user groups that can access this replication group. Defaults to None.
        log_delivery_configurations(List, optional): Specifies the destination, format and type of the logs. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.elasticache.replication_group.init.modify(
                ctx, replication_group_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.elasticache.replication_group.init.modify replication_group_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="elasticache",
        operation="modify_replication_group",
        op_kwargs={
            "ReplicationGroupId": replication_group_id,
            "ReplicationGroupDescription": replication_group_description,
            "PrimaryClusterId": primary_cluster_id,
            "SnapshottingClusterId": snapshotting_cluster_id,
            "AutomaticFailoverEnabled": automatic_failover_enabled,
            "MultiAZEnabled": multi_az_enabled,
            "NodeGroupId": node_group_id,
            "CacheSecurityGroupNames": cache_security_group_names,
            "SecurityGroupIds": security_group_ids,
            "PreferredMaintenanceWindow": preferred_maintenance_window,
            "NotificationTopicArn": notification_topic_arn,
            "CacheParameterGroupName": cache_parameter_group_name,
            "NotificationTopicStatus": notification_topic_status,
            "ApplyImmediately": apply_immediately,
            "EngineVersion": engine_version,
            "AutoMinorVersionUpgrade": auto_minor_version_upgrade,
            "SnapshotRetentionLimit": snapshot_retention_limit,
            "SnapshotWindow": snapshot_window,
            "CacheNodeType": cache_node_type,
            "AuthToken": auth_token,
            "AuthTokenUpdateStrategy": auth_token_update_strategy,
            "UserGroupIdsToAdd": user_group_ids_to_add,
            "UserGroupIdsToRemove": user_group_ids_to_remove,
            "RemoveUserGroups": remove_user_groups,
            "LogDeliveryConfigurations": log_delivery_configurations,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the SMS sandbox status for the calling account in the target Region. When you start using Amazon SNS to send SMS messages, your account is in the SMS sandbox. The SMS sandbox provides a safe environment for you to try Amazon SNS features without risking your reputation as an SMS sender. While your account is in the SMS sandbox, you can use all of the features of Amazon SNS. However, you can send SMS messages only to verified destination phone numbers. For more information, including how to move out of the sandbox to send messages without restrictions, see SMS sandbox in the Amazon SNS Developer Guide.


    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.sns.sms.sandbox.account_status.get(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.sns.sms.sandbox.account_status.get
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="sns",
        operation="get_sms_sandbox_account_status",
        op_kwargs={},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe(
    hub, ctx, firewall_arn: str = None, firewall_name: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Returns the logging configuration for the specified firewall.

    Args:
        firewall_arn(str, optional): The Amazon Resource Name (ARN) of the firewall. You must specify the ARN or the name, and you can specify both. . Defaults to None.
        firewall_name(str, optional): The descriptive name of the firewall. You can't change the name of a firewall after you create it. You must specify the ARN or the name, and you can specify both. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.network_firewall.logging_configuration.describe(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.network_firewall.logging_configuration.describe
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="network-firewall",
        operation="describe_logging_configuration",
        op_kwargs={"FirewallArn": firewall_arn, "FirewallName": firewall_name},
    )


async def update(
    hub,
    ctx,
    firewall_arn: str = None,
    firewall_name: str = None,
    logging_configuration: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Sets the logging configuration for the specified firewall.  To change the logging configuration, retrieve the LoggingConfiguration by calling DescribeLoggingConfiguration, then change it and provide the modified object to this update call. You must change the logging configuration one LogDestinationConfig at a time inside the retrieved LoggingConfiguration object.  You can perform only one of the following actions in any call to UpdateLoggingConfiguration:    Create a new log destination object by adding a single LogDestinationConfig array element to LogDestinationConfigs.   Delete a log destination object by removing a single LogDestinationConfig array element from LogDestinationConfigs.   Change the LogDestination setting in a single LogDestinationConfig array element.   You can't change the LogDestinationType or LogType in a LogDestinationConfig. To change these settings, delete the existing LogDestinationConfig object and create a new one, using two separate calls to this update operation.

    Args:
        firewall_arn(str, optional): The Amazon Resource Name (ARN) of the firewall. You must specify the ARN or the name, and you can specify both. . Defaults to None.
        firewall_name(str, optional): The descriptive name of the firewall. You can't change the name of a firewall after you create it. You must specify the ARN or the name, and you can specify both. . Defaults to None.
        logging_configuration(Dict, optional): Defines how Network Firewall performs logging for a firewall. If you omit this setting, Network Firewall disables logging for the firewall. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.network_firewall.logging_configuration.update(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.network_firewall.logging_configuration.update
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="network-firewall",
        operation="update_logging_configuration",
        op_kwargs={
            "FirewallArn": firewall_arn,
            "FirewallName": firewall_name,
            "LoggingConfiguration": logging_configuration,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__func_alias__ = {"list_": "list"}


async def list_(
    hub,
    ctx,
    certificate_authority_arn: str,
    next_token: str = None,
    max_results: int = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the tags, if any, that are associated with your private CA or one that has been shared with you. Tags are labels that you can use to identify and organize your CAs. Each tag consists of a key and an optional value. Call the TagCertificateAuthority action to add one or more tags to your CA. Call the UntagCertificateAuthority action to remove tags.

    Args:
        certificate_authority_arn(str): The Amazon Resource Name (ARN) that was returned when you called the CreateCertificateAuthority action. This must be of the form:   arn:aws:acm-pca:region:account:certificate-authority/12345678-1234-1234-1234-123456789012  .
        next_token(str, optional): Use this parameter when paginating results in a subsequent request after you receive a response with truncated results. Set it to the value of NextToken from the response you just received. Defaults to None.
        max_results(int, optional): Use this parameter when paginating results to specify the maximum number of items to return in the response. If additional items exist beyond the number you specify, the NextToken element is sent in the response. Use this NextToken value in a subsequent request to retrieve additional items. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.acm_pca.tags.list(ctx, certificate_authority_arn=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.acm_pca.tags.list certificate_authority_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="acm-pca",
        operation="list_tags",
        op_kwargs={
            "CertificateAuthorityArn": certificate_authority_arn,
            "NextToken": next_token,
            "MaxResults": max_results,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def invite(hub, ctx, target: Dict, notes: str = None, tags: List = None) -> Dict:
    r"""
    **Autogenerated function**

    Sends an invitation to another account to join your organization as a member account. AWS Organizations sends email on your behalf to the email address that is associated with the other account's owner. The invitation is implemented as a Handshake whose details are in the response.    You can invite AWS accounts only from the same seller as the management account. For example, if your organization's management account was created by Amazon Internet Services Pvt. Ltd (AISPL), an AWS seller in India, you can invite only other AISPL accounts to your organization. You can't combine accounts from AISPL and AWS or from any other AWS seller. For more information, see Consolidated Billing in India.   If you receive an exception that indicates that you exceeded your account limits for the organization or that the operation failed because your organization is still initializing, wait one hour and then try again. If the error persists after an hour, contact AWS Support.    If the request includes tags, then the requester must have the organizations:TagResource permission. This operation can be called only from the organization's management account.

    Args:
        target(Dict): The identifier (ID) of the AWS account that you want to invite to join your organization. This is a JSON object that contains the following elements:  { "Type": "ACCOUNT", "Id": "< account id number >" }  If you use the AWS CLI, you can submit this as a single string, similar to the following example:  --target Id=123456789012,Type=ACCOUNT  If you specify "Type": "ACCOUNT", you must provide the AWS account ID number as the Id. If you specify "Type": "EMAIL", you must specify the email address that is associated with the account.  --target Id=diego@example.com,Type=EMAIL .
        notes(str, optional): Additional information that you want to include in the generated email to the recipient account owner. Defaults to None.
        tags(List, optional): A list of tags that you want to attach to the account when it becomes a member of the organization. For each tag in the list, you must specify both a tag key and a value. You can set the value to an empty string, but you can't set it to null. For more information about tagging, see Tagging AWS Organizations resources in the AWS Organizations User Guide.  Any tags in the request are checked for compliance with any applicable tag policies when the request is made. The request is rejected if the tags in the request don't match the requirements of the policy at that time. Tag policy compliance is  not  checked again when the invitation is accepted and the tags are actually attached to the account. That means that if the tag policy changes between the invitation and the acceptance, then that tags could potentially be non-compliant.   If any one of the tags is invalid or if you exceed the allowed number of tags for an account, then the entire request fails and invitations are not sent. . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.organization.account.to_organization.invite(ctx, target=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.organization.account.to_organization.invite target=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="organizations",
        operation="invite_account_to_organization",
        op_kwargs={"Target": target, "Notes": notes, "Tags": tags},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def start(hub, ctx, meeting_id: str, transcription_configuration: Dict) -> Dict:
    r"""
    **Autogenerated function**

    Start transcription for the specified meetingId.

    Args:
        meeting_id(str): The unique ID of the meeting being transcribed.
        transcription_configuration(Dict): The configuration for the current transcription operation. Must contain EngineTranscribeSettings or EngineTranscribeMedicalSettings.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.meeting.transcription.start(
                ctx, meeting_id=value, transcription_configuration=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.meeting.transcription.start meeting_id=value, transcription_configuration=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="start_meeting_transcription",
        op_kwargs={
            "MeetingId": meeting_id,
            "TranscriptionConfiguration": transcription_configuration,
        },
    )


async def stop(hub, ctx, meeting_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Stops transcription for the specified meetingId.

    Args:
        meeting_id(str): The unique ID of the meeting for which you stop transcription.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.chime.meeting.transcription.stop(ctx, meeting_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.chime.meeting.transcription.stop meeting_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="chime",
        operation="stop_meeting_transcription",
        op_kwargs={"MeetingId": meeting_id},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    traffic_mirror_filter_id: str,
    traffic_direction: str,
    rule_number: int,
    rule_action: str,
    destination_cidr_block: str,
    source_cidr_block: str,
    destination_port_range: Dict = None,
    source_port_range: Dict = None,
    protocol: int = None,
    description: str = None,
    dry_run: bool = None,
    client_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a Traffic Mirror filter rule.  A Traffic Mirror rule defines the Traffic Mirror source traffic to mirror. You need the Traffic Mirror filter ID when you create the rule.

    Args:
        traffic_mirror_filter_id(str): The ID of the filter that this rule is associated with.
        traffic_direction(str): The type of traffic (ingress | egress).
        rule_number(int): The number of the Traffic Mirror rule. This number must be unique for each Traffic Mirror rule in a given direction. The rules are processed in ascending order by rule number.
        rule_action(str): The action to take (accept | reject) on the filtered traffic.
        destination_port_range(Dict, optional): The destination port range. Defaults to None.
        source_port_range(Dict, optional): The source port range. Defaults to None.
        protocol(int, optional): The protocol, for example UDP, to assign to the Traffic Mirror rule. For information about the protocol value, see Protocol Numbers on the Internet Assigned Numbers Authority (IANA) website. Defaults to None.
        destination_cidr_block(str): The destination CIDR block to assign to the Traffic Mirror rule.
        source_cidr_block(str): The source CIDR block to assign to the Traffic Mirror rule.
        description(str, optional): The description of the Traffic Mirror rule. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.
        client_token(str, optional): Unique, case-sensitive identifier that you provide to ensure the idempotency of the request. For more information, see How to Ensure Idempotency. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.traffic_mirror.filter_.rule.create(
                ctx,
                traffic_mirror_filter_id=value,
                traffic_direction=value,
                rule_number=value,
                rule_action=value,
                destination_cidr_block=value,
                source_cidr_block=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.traffic_mirror.filter_.rule.create traffic_mirror_filter_id=value, traffic_direction=value, rule_number=value, rule_action=value, destination_cidr_block=value, source_cidr_block=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="create_traffic_mirror_filter_rule",
        op_kwargs={
            "TrafficMirrorFilterId": traffic_mirror_filter_id,
            "TrafficDirection": traffic_direction,
            "RuleNumber": rule_number,
            "RuleAction": rule_action,
            "DestinationPortRange": destination_port_range,
            "SourcePortRange": source_port_range,
            "Protocol": protocol,
            "DestinationCidrBlock": destination_cidr_block,
            "SourceCidrBlock": source_cidr_block,
            "Description": description,
            "DryRun": dry_run,
            "ClientToken": client_token,
        },
    )


async def delete(
    hub, ctx, traffic_mirror_filter_rule_id: str, dry_run: bool = None
) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the specified Traffic Mirror rule.

    Args:
        traffic_mirror_filter_rule_id(str): The ID of the Traffic Mirror rule.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.traffic_mirror.filter_.rule.delete(
                ctx, traffic_mirror_filter_rule_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.traffic_mirror.filter_.rule.delete traffic_mirror_filter_rule_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="delete_traffic_mirror_filter_rule",
        op_kwargs={
            "TrafficMirrorFilterRuleId": traffic_mirror_filter_rule_id,
            "DryRun": dry_run,
        },
    )


async def modify(
    hub,
    ctx,
    traffic_mirror_filter_rule_id: str,
    traffic_direction: str = None,
    rule_number: int = None,
    rule_action: str = None,
    destination_port_range: Dict = None,
    source_port_range: Dict = None,
    protocol: int = None,
    destination_cidr_block: str = None,
    source_cidr_block: str = None,
    description: str = None,
    remove_fields: List = None,
    dry_run: bool = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Modifies the specified Traffic Mirror rule.  DestinationCidrBlock and SourceCidrBlock must both be an IPv4 range or an IPv6 range.

    Args:
        traffic_mirror_filter_rule_id(str): The ID of the Traffic Mirror rule.
        traffic_direction(str, optional): The type of traffic (ingress | egress) to assign to the rule. Defaults to None.
        rule_number(int, optional): The number of the Traffic Mirror rule. This number must be unique for each Traffic Mirror rule in a given direction. The rules are processed in ascending order by rule number. Defaults to None.
        rule_action(str, optional): The action to assign to the rule. Defaults to None.
        destination_port_range(Dict, optional): The destination ports that are associated with the Traffic Mirror rule. Defaults to None.
        source_port_range(Dict, optional): The port range to assign to the Traffic Mirror rule. Defaults to None.
        protocol(int, optional): The protocol, for example TCP, to assign to the Traffic Mirror rule. Defaults to None.
        destination_cidr_block(str, optional): The destination CIDR block to assign to the Traffic Mirror rule. Defaults to None.
        source_cidr_block(str, optional): The source CIDR block to assign to the Traffic Mirror rule. Defaults to None.
        description(str, optional): The description to assign to the Traffic Mirror rule. Defaults to None.
        remove_fields(List, optional): The properties that you want to remove from the Traffic Mirror filter rule. When you remove a property from a Traffic Mirror filter rule, the property is set to the default. Defaults to None.
        dry_run(bool, optional): Checks whether you have the required permissions for the action, without actually making the request, and provides an error response. If you have the required permissions, the error response is DryRunOperation. Otherwise, it is UnauthorizedOperation. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ec2.traffic_mirror.filter_.rule.modify(
                ctx, traffic_mirror_filter_rule_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ec2.traffic_mirror.filter_.rule.modify traffic_mirror_filter_rule_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ec2",
        operation="modify_traffic_mirror_filter_rule",
        op_kwargs={
            "TrafficMirrorFilterRuleId": traffic_mirror_filter_rule_id,
            "TrafficDirection": traffic_direction,
            "RuleNumber": rule_number,
            "RuleAction": rule_action,
            "DestinationPortRange": destination_port_range,
            "SourcePortRange": source_port_range,
            "Protocol": protocol,
            "DestinationCidrBlock": destination_cidr_block,
            "SourceCidrBlock": source_cidr_block,
            "Description": description,
            "RemoveFields": remove_fields,
            "DryRun": dry_run,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def batch(
    hub,
    ctx,
    channel_ids: List = None,
    input_ids: List = None,
    input_security_group_ids: List = None,
    multiplex_ids: List = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Starts delete of resources.

    Args:
        channel_ids(List, optional): List of channel IDs. Defaults to None.
        input_ids(List, optional): List of input IDs. Defaults to None.
        input_security_group_ids(List, optional): List of input security group IDs. Defaults to None.
        multiplex_ids(List, optional): List of multiplex IDs. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.medialive.delete.batch(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.medialive.delete.batch
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="medialive",
        operation="batch_delete",
        op_kwargs={
            "ChannelIds": channel_ids,
            "InputIds": input_ids,
            "InputSecurityGroupIds": input_security_group_ids,
            "MultiplexIds": multiplex_ids,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, profiling_group_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Get the current configuration for anomaly notifications for a profiling group.

    Args:
        profiling_group_name(str): The name of the profiling group we want to get the notification configuration for.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.codeguruprofiler.notification.configuration.get(
                ctx, profiling_group_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.codeguruprofiler.notification.configuration.get profiling_group_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="codeguruprofiler",
        operation="get_notification_configuration",
        op_kwargs={"profilingGroupName": profiling_group_name},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def get(hub, ctx, project_name: str, placement_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns an object enumerating the devices in a placement.

    Args:
        project_name(str): The name of the project containing the placement.
        placement_name(str): The name of the placement to get the devices from.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iot1click_project.device.in_placement.get(
                ctx, project_name=value, placement_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iot1click_project.device.in_placement.get project_name=value, placement_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iot1click-projects",
        operation="get_devices_in_placement",
        op_kwargs={"projectName": project_name, "placementName": placement_name},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def get(
    hub,
    ctx,
    bucket_name: str,
    metric_name: str,
    start_time: str,
    end_time: str,
    period: int,
    statistics: List,
    unit: str,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns the data points of a specific metric for an Amazon Lightsail bucket. Metrics report the utilization of a bucket. View and collect metric data regularly to monitor the number of objects stored in a bucket (including object versions) and the storage space used by those objects.

    Args:
        bucket_name(str): The name of the bucket for which to get metric data.
        metric_name(str): The metric for which you want to return information. Valid bucket metric names are listed below, along with the most useful statistics to include in your request, and the published unit value.  These bucket metrics are reported once per day.      BucketSizeBytes  - The amount of data in bytes stored in a bucket. This value is calculated by summing the size of all objects in the bucket (including object versions), including the size of all parts for all incomplete multipart uploads to the bucket. Statistics: The most useful statistic is Maximum. Unit: The published unit is Bytes.     NumberOfObjects  - The total number of objects stored in a bucket. This value is calculated by counting all objects in the bucket (including object versions) and the total number of parts for all incomplete multipart uploads to the bucket. Statistics: The most useful statistic is Average. Unit: The published unit is Count.  .
        start_time(str): The timestamp indicating the earliest data to be returned.
        end_time(str): The timestamp indicating the latest data to be returned.
        period(int): The granularity, in seconds, of the returned data points.  Bucket storage metrics are reported once per day. Therefore, you should specify a period of 86400 seconds, which is the number of seconds in a day. .
        statistics(List): The statistic for the metric. The following statistics are available:    Minimum - The lowest value observed during the specified period. Use this value to determine low volumes of activity for your application.    Maximum - The highest value observed during the specified period. Use this value to determine high volumes of activity for your application.    Sum - The sum of all values submitted for the matching metric. You can use this statistic to determine the total volume of a metric.    Average - The value of Sum / SampleCount during the specified period. By comparing this statistic with the Minimum and Maximum values, you can determine the full scope of a metric and how close the average use is to the Minimum and Maximum values. This comparison helps you to know when to increase or decrease your resources.    SampleCount - The count, or number, of data points used for the statistical calculation.  .
        unit(str): The unit for the metric data request. Valid units depend on the metric data being requested. For the valid units with each available metric, see the metricName parameter.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.lightsail.bucket.metric_data.get(
                ctx,
                bucket_name=value,
                metric_name=value,
                start_time=value,
                end_time=value,
                period=value,
                statistics=value,
                unit=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.lightsail.bucket.metric_data.get bucket_name=value, metric_name=value, start_time=value, end_time=value, period=value, statistics=value, unit=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="lightsail",
        operation="get_bucket_metric_data",
        op_kwargs={
            "bucketName": bucket_name,
            "metricName": metric_name,
            "startTime": start_time,
            "endTime": end_time,
            "period": period,
            "statistics": statistics,
            "unit": unit,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    role: str,
    server_id: str,
    external_id: str,
    home_directory: str = None,
    home_directory_type: str = None,
    home_directory_mappings: List = None,
    policy: str = None,
    posix_profile: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Used by administrators to choose which groups in the directory should have access to upload and download files over the enabled protocols using Amazon Web Services Transfer Family. For example, a Microsoft Active Directory might contain 50,000 users, but only a small fraction might need the ability to transfer files to the server. An administrator can use CreateAccess to limit the access to the correct set of users who need this ability.

    Args:
        home_directory(str, optional): The landing directory (folder) for a user when they log in to the server using the client. A HomeDirectory example is /bucket_name/home/mydirectory. Defaults to None.
        home_directory_type(str, optional): The type of landing directory (folder) you want your users' home directory to be when they log into the server. If you set it to PATH, the user will see the absolute Amazon S3 bucket or EFS paths as is in their file transfer protocol clients. If you set it LOGICAL, you will need to provide mappings in the HomeDirectoryMappings for how you want to make Amazon S3 or EFS paths visible to your users. Defaults to None.
        home_directory_mappings(List, optional): Logical directory mappings that specify what Amazon S3 or Amazon EFS paths and keys should be visible to your user and how you want to make them visible. You must specify the Entry and Target pair, where Entry shows how the path is made visible and Target is the actual Amazon S3 or Amazon EFS path. If you only specify a target, it is displayed as is. You also must ensure that your Amazon Web Services Identity and Access Management (IAM) role provides access to paths in Target. This value can only be set when HomeDirectoryType is set to LOGICAL. The following is an Entry and Target pair example.  [ { "Entry": "your-personal-report.pdf", "Target": "/bucket3/customized-reports/${transfer:UserName}.pdf" } ]  In most cases, you can use this value instead of the scope-down policy to lock down your user to the designated home directory ("chroot"). To do this, you can set Entry to / and set Target to the HomeDirectory parameter value. The following is an Entry and Target pair example for chroot.  [ { "Entry:": "/", "Target": "/bucket_name/home/mydirectory" } ]   If the target of a logical directory entry does not exist in Amazon S3 or EFS, the entry is ignored. As a workaround, you can use the Amazon S3 API or EFS API to create 0 byte objects as place holders for your directory. If using the CLI, use the s3api or efsapi call instead of s3 or efs so you can use the put-object operation. For example, you use the following: aws s3api put-object --bucket bucketname --key path/to/folder/. Make sure that the end of the key name ends in a / for it to be considered a folder. . Defaults to None.
        policy(str, optional): A scope-down policy for your user so that you can use the same IAM role across multiple users. This policy scopes down user access to portions of their Amazon S3 bucket. Variables that you can use inside this policy include ${Transfer:UserName}, ${Transfer:HomeDirectory}, and ${Transfer:HomeBucket}.  This only applies when domain of ServerId is S3. Amazon EFS does not use scope-down policies. For scope-down policies, Amazon Web Services Transfer Family stores the policy as a JSON blob, instead of the Amazon Resource Name (ARN) of the policy. You save the policy as a JSON blob and pass it in the Policy argument. For an example of a scope-down policy, see Example scope-down policy. For more information, see AssumeRole in the Amazon Web Services Security Token Service API Reference. . Defaults to None.
        posix_profile(Dict, optional): The full POSIX identity, including user ID (Uid), group ID (Gid), and any secondary groups IDs (SecondaryGids), that controls your users' access to your Amazon EFS file systems. The POSIX permissions that are set on files and directories in your file system determine the level of access your users get when transferring files into and out of your Amazon EFS file systems. Defaults to None.
        role(str): Specifies the Amazon Resource Name (ARN) of the IAM role that controls your users' access to your Amazon S3 bucket or EFS file system. The policies attached to this role determine the level of access that you want to provide your users when transferring files into and out of your Amazon S3 bucket or EFS file system. The IAM role should also contain a trust relationship that allows the server to access your resources when servicing your users' transfer requests.
        server_id(str): A system-assigned unique identifier for a server instance. This is the specific server that you added your user to.
        external_id(str): A unique identifier that is required to identify specific groups within your directory. The users of the group that you associate have access to your Amazon S3 or Amazon EFS resources over the enabled protocols using Amazon Web Services Transfer Family. If you know the group name, you can view the SID values by running the following command using Windows PowerShell.  Get-ADGroup -Filter {samAccountName -like "YourGroupName*"} -Properties * | Select SamAccountName,ObjectSid  In that command, replace YourGroupName with the name of your Active Directory group. The regex used to validate this parameter is a string of characters consisting of uppercase and lowercase alphanumeric characters with no spaces. You can also include underscores or any of the following characters: =,.@:/-.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.transfer.access.create(
                ctx, role=value, server_id=value, external_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.transfer.access.create role=value, server_id=value, external_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="transfer",
        operation="create_access",
        op_kwargs={
            "HomeDirectory": home_directory,
            "HomeDirectoryType": home_directory_type,
            "HomeDirectoryMappings": home_directory_mappings,
            "Policy": policy,
            "PosixProfile": posix_profile,
            "Role": role,
            "ServerId": server_id,
            "ExternalId": external_id,
        },
    )


async def delete(hub, ctx, server_id: str, external_id: str) -> None:
    r"""
    **Autogenerated function**

    Allows you to delete the access specified in the ServerID and ExternalID parameters.

    Args:
        server_id(str): A system-assigned unique identifier for a server that has this user assigned.
        external_id(str): A unique identifier that is required to identify specific groups within your directory. The users of the group that you associate have access to your Amazon S3 or Amazon EFS resources over the enabled protocols using Amazon Web Services Transfer Family. If you know the group name, you can view the SID values by running the following command using Windows PowerShell.  Get-ADGroup -Filter {samAccountName -like "YourGroupName*"} -Properties * | Select SamAccountName,ObjectSid  In that command, replace YourGroupName with the name of your Active Directory group. The regex used to validate this parameter is a string of characters consisting of uppercase and lowercase alphanumeric characters with no spaces. You can also include underscores or any of the following characters: =,.@:/-.
    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.transfer.access.delete(ctx, server_id=value, external_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.transfer.access.delete server_id=value, external_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="transfer",
        operation="delete_access",
        op_kwargs={"ServerId": server_id, "ExternalId": external_id},
    )


async def describe(hub, ctx, server_id: str, external_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Describes the access that is assigned to the specific file transfer protocol-enabled server, as identified by its ServerId property and its ExternalID. The response from this call returns the properties of the access that is associated with the ServerId value that was specified.

    Args:
        server_id(str): A system-assigned unique identifier for a server that has this access assigned.
        external_id(str): A unique identifier that is required to identify specific groups within your directory. The users of the group that you associate have access to your Amazon S3 or Amazon EFS resources over the enabled protocols using Amazon Web Services Transfer Family. If you know the group name, you can view the SID values by running the following command using Windows PowerShell.  Get-ADGroup -Filter {samAccountName -like "YourGroupName*"} -Properties * | Select SamAccountName,ObjectSid  In that command, replace YourGroupName with the name of your Active Directory group. The regex used to validate this parameter is a string of characters consisting of uppercase and lowercase alphanumeric characters with no spaces. You can also include underscores or any of the following characters: =,.@:/-.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.transfer.access.describe(
                ctx, server_id=value, external_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.transfer.access.describe server_id=value, external_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="transfer",
        operation="describe_access",
        op_kwargs={"ServerId": server_id, "ExternalId": external_id},
    )


async def list_all(
    hub, ctx, server_id: str, max_results: int = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Lists the details for all the accesses you have on your server.

    Args:
        max_results(int, optional): Specifies the maximum number of access SIDs to return. Defaults to None.
        next_token(str, optional): When you can get additional results from the ListAccesses call, a NextToken parameter is returned in the output. You can then pass in a subsequent command to the NextToken parameter to continue listing additional accesses. Defaults to None.
        server_id(str): A system-assigned unique identifier for a server that has users assigned to it.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.transfer.access.list_all(ctx, server_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.transfer.access.list_all server_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="transfer",
        operation="list_accesses",
        op_kwargs={
            "MaxResults": max_results,
            "NextToken": next_token,
            "ServerId": server_id,
        },
    )


async def update(
    hub,
    ctx,
    server_id: str,
    external_id: str,
    home_directory: str = None,
    home_directory_type: str = None,
    home_directory_mappings: List = None,
    policy: str = None,
    posix_profile: Dict = None,
    role: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Allows you to update parameters for the access specified in the ServerID and ExternalID parameters.

    Args:
        home_directory(str, optional): The landing directory (folder) for a user when they log in to the server using the client. A HomeDirectory example is /bucket_name/home/mydirectory. Defaults to None.
        home_directory_type(str, optional): The type of landing directory (folder) you want your users' home directory to be when they log into the server. If you set it to PATH, the user will see the absolute Amazon S3 bucket or EFS paths as is in their file transfer protocol clients. If you set it LOGICAL, you will need to provide mappings in the HomeDirectoryMappings for how you want to make Amazon S3 or EFS paths visible to your users. Defaults to None.
        home_directory_mappings(List, optional): Logical directory mappings that specify what Amazon S3 or Amazon EFS paths and keys should be visible to your user and how you want to make them visible. You must specify the Entry and Target pair, where Entry shows how the path is made visible and Target is the actual Amazon S3 or Amazon EFS path. If you only specify a target, it is displayed as is. You also must ensure that your Amazon Web Services Identity and Access Management (IAM) role provides access to paths in Target. This value can only be set when HomeDirectoryType is set to LOGICAL. The following is an Entry and Target pair example.  [ { "Entry": "your-personal-report.pdf", "Target": "/bucket3/customized-reports/${transfer:UserName}.pdf" } ]  In most cases, you can use this value instead of the scope-down policy to lock down your user to the designated home directory ("chroot"). To do this, you can set Entry to / and set Target to the HomeDirectory parameter value. The following is an Entry and Target pair example for chroot.  [ { "Entry:": "/", "Target": "/bucket_name/home/mydirectory" } ]   If the target of a logical directory entry does not exist in Amazon S3 or EFS, the entry is ignored. As a workaround, you can use the Amazon S3 API or EFS API to create 0 byte objects as place holders for your directory. If using the CLI, use the s3api or efsapi call instead of s3 or efs so you can use the put-object operation. For example, you use the following: aws s3api put-object --bucket bucketname --key path/to/folder/. Make sure that the end of the key name ends in a / for it to be considered a folder. . Defaults to None.
        policy(str, optional): A scope-down policy for your user so that you can use the same IAM role across multiple users. This policy scopes down user access to portions of their Amazon S3 bucket. Variables that you can use inside this policy include ${Transfer:UserName}, ${Transfer:HomeDirectory}, and ${Transfer:HomeBucket}.  This only applies when domain of ServerId is S3. Amazon EFS does not use scope down policy. For scope-down policies, Amazon Web ServicesTransfer Family stores the policy as a JSON blob, instead of the Amazon Resource Name (ARN) of the policy. You save the policy as a JSON blob and pass it in the Policy argument. For an example of a scope-down policy, see Example scope-down policy. For more information, see AssumeRole in the Amazon Web ServicesSecurity Token Service API Reference. . Defaults to None.
        posix_profile(Dict, optional): The full POSIX identity, including user ID (Uid), group ID (Gid), and any secondary groups IDs (SecondaryGids), that controls your users' access to your Amazon EFS file systems. The POSIX permissions that are set on files and directories in your file system determine the level of access your users get when transferring files into and out of your Amazon EFS file systems. Defaults to None.
        role(str, optional): Specifies the Amazon Resource Name (ARN) of the IAM role that controls your users' access to your Amazon S3 bucket or EFS file system. The policies attached to this role determine the level of access that you want to provide your users when transferring files into and out of your Amazon S3 bucket or EFS file system. The IAM role should also contain a trust relationship that allows the server to access your resources when servicing your users' transfer requests. Defaults to None.
        server_id(str): A system-assigned unique identifier for a server instance. This is the specific server that you added your user to.
        external_id(str): A unique identifier that is required to identify specific groups within your directory. The users of the group that you associate have access to your Amazon S3 or Amazon EFS resources over the enabled protocols using Amazon Web Services Transfer Family. If you know the group name, you can view the SID values by running the following command using Windows PowerShell.  Get-ADGroup -Filter {samAccountName -like "YourGroupName*"} -Properties * | Select SamAccountName,ObjectSid  In that command, replace YourGroupName with the name of your Active Directory group. The regex used to validate this parameter is a string of characters consisting of uppercase and lowercase alphanumeric characters with no spaces. You can also include underscores or any of the following characters: =,.@:/-.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.transfer.access.update(ctx, server_id=value, external_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.transfer.access.update server_id=value, external_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="transfer",
        operation="update_access",
        op_kwargs={
            "HomeDirectory": home_directory,
            "HomeDirectoryType": home_directory_type,
            "HomeDirectoryMappings": home_directory_mappings,
            "Policy": policy,
            "PosixProfile": posix_profile,
            "Role": role,
            "ServerId": server_id,
            "ExternalId": external_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub,
    ctx,
    portal_name: str,
    portal_contact_email: str,
    role_arn: str,
    portal_description: str = None,
    client_token: str = None,
    portal_logo_image_file: Dict = None,
    tags: Dict = None,
    portal_auth_mode: str = None,
    notification_sender_email: str = None,
    alarms: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a portal, which can contain projects and dashboards. IoT SiteWise Monitor uses Amazon Web Services SSO or IAM to authenticate portal users and manage user permissions.  Before you can sign in to a new portal, you must add at least one identity to that portal. For more information, see Adding or removing portal administrators in the IoT SiteWise User Guide.

    Args:
        portal_name(str): A friendly name for the portal.
        portal_description(str, optional): A description for the portal. Defaults to None.
        portal_contact_email(str): The Amazon Web Services administrator's contact email address.
        client_token(str, optional): A unique case-sensitive identifier that you can provide to ensure the idempotency of the request. Don't reuse this client token if a new idempotent request is required. Defaults to None.
        portal_logo_image_file(Dict, optional): A logo image to display in the portal. Upload a square, high-resolution image. The image is displayed on a dark background. Defaults to None.
        role_arn(str): The ARN of a service role that allows the portal's users to access your IoT SiteWise resources on your behalf. For more information, see Using service roles for IoT SiteWise Monitor in the IoT SiteWise User Guide.
        tags(Dict, optional): A list of key-value pairs that contain metadata for the portal. For more information, see Tagging your IoT SiteWise resources in the IoT SiteWise User Guide. Defaults to None.
        portal_auth_mode(str, optional): The service to use to authenticate users to the portal. Choose from the following options:    SSO – The portal uses Amazon Web Services Single Sign On to authenticate users and manage user permissions. Before you can create a portal that uses Amazon Web Services SSO, you must enable Amazon Web Services SSO. For more information, see Enabling Amazon Web Services SSO in the IoT SiteWise User Guide. This option is only available in Amazon Web Services Regions other than the China Regions.    IAM – The portal uses Identity and Access Management to authenticate users and manage user permissions. This option is only available in the China Regions.   You can't change this value after you create a portal. Default: SSO . Defaults to None.
        notification_sender_email(str, optional): The email address that sends alarm notifications.  If you use the IoT Events managed Lambda function to manage your emails, you must verify the sender email address in Amazon SES. . Defaults to None.
        alarms(Dict, optional): Contains the configuration information of an alarm created in an IoT SiteWise Monitor portal. You can use the alarm to monitor an asset property and get notified when the asset property value is outside a specified range. For more information, see Monitoring with alarms in the IoT SiteWise Application Guide. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.portal.create(
                ctx, portal_name=value, portal_contact_email=value, role_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.portal.create portal_name=value, portal_contact_email=value, role_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="create_portal",
        op_kwargs={
            "portalName": portal_name,
            "portalDescription": portal_description,
            "portalContactEmail": portal_contact_email,
            "clientToken": client_token,
            "portalLogoImageFile": portal_logo_image_file,
            "roleArn": role_arn,
            "tags": tags,
            "portalAuthMode": portal_auth_mode,
            "notificationSenderEmail": notification_sender_email,
            "alarms": alarms,
        },
    )


async def delete(hub, ctx, portal_id: str, client_token: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Deletes a portal from IoT SiteWise Monitor.

    Args:
        portal_id(str): The ID of the portal to delete.
        client_token(str, optional): A unique case-sensitive identifier that you can provide to ensure the idempotency of the request. Don't reuse this client token if a new idempotent request is required. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.portal.delete(ctx, portal_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.portal.delete portal_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="delete_portal",
        op_kwargs={"portalId": portal_id, "clientToken": client_token},
    )


async def describe(hub, ctx, portal_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves information about a portal.

    Args:
        portal_id(str): The ID of the portal.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.portal.describe(ctx, portal_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.portal.describe portal_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="describe_portal",
        op_kwargs={"portalId": portal_id},
    )


async def list_all(hub, ctx, next_token: str = None, max_results: int = None) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves a paginated list of IoT SiteWise Monitor portals.

    Args:
        next_token(str, optional): The token to be used for the next set of paginated results. Defaults to None.
        max_results(int, optional): The maximum number of results to return for each paginated request. Default: 50. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.portal.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.portal.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="list_portals",
        op_kwargs={"nextToken": next_token, "maxResults": max_results},
    )


async def update(
    hub,
    ctx,
    portal_id: str,
    portal_name: str,
    portal_contact_email: str,
    role_arn: str,
    portal_description: str = None,
    portal_logo_image: Dict = None,
    client_token: str = None,
    notification_sender_email: str = None,
    alarms: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Updates an IoT SiteWise Monitor portal.

    Args:
        portal_id(str): The ID of the portal to update.
        portal_name(str): A new friendly name for the portal.
        portal_description(str, optional): A new description for the portal. Defaults to None.
        portal_contact_email(str): The Amazon Web Services administrator's contact email address.
        portal_logo_image(Dict, optional): Contains an image that is one of the following:   An image file. Choose this option to upload a new image.   The ID of an existing image. Choose this option to keep an existing image.  . Defaults to None.
        role_arn(str): The ARN of a service role that allows the portal's users to access your IoT SiteWise resources on your behalf. For more information, see Using service roles for IoT SiteWise Monitor in the IoT SiteWise User Guide.
        client_token(str, optional): A unique case-sensitive identifier that you can provide to ensure the idempotency of the request. Don't reuse this client token if a new idempotent request is required. Defaults to None.
        notification_sender_email(str, optional): The email address that sends alarm notifications. Defaults to None.
        alarms(Dict, optional): Contains the configuration information of an alarm created in an IoT SiteWise Monitor portal. You can use the alarm to monitor an asset property and get notified when the asset property value is outside a specified range. For more information, see Monitoring with alarms in the IoT SiteWise Application Guide. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.iotsitewise.portal.update(
                ctx, portal_id=value, portal_name=value, portal_contact_email=value, role_arn=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.iotsitewise.portal.update portal_id=value, portal_name=value, portal_contact_email=value, role_arn=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="iotsitewise",
        operation="update_portal",
        op_kwargs={
            "portalId": portal_id,
            "portalName": portal_name,
            "portalDescription": portal_description,
            "portalContactEmail": portal_contact_email,
            "portalLogoImage": portal_logo_image,
            "roleArn": role_arn,
            "clientToken": client_token,
            "notificationSenderEmail": notification_sender_email,
            "alarms": alarms,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(
    hub,
    ctx,
    multiplex_id: str,
    multiplex_program_settings: Dict,
    program_name: str,
    request_id: str,
) -> Dict:
    r"""
        **Autogenerated function**

        Create a new program in the multiplex.

        Args:
            multiplex_id(str): ID of the multiplex where the program is to be created.
            multiplex_program_settings(Dict): The settings for this multiplex program.
            program_name(str): Name of multiplex program.
            request_id(str): Unique request ID. This prevents retries from creating multiple
    resources.
    .

        Returns:
            Dict

        Examples:
            Call from code:

            .. code-block:: python

                await hub.exec.aws_auto.medialive.multiplex.program.create(
                    ctx,
                    multiplex_id=value,
                    multiplex_program_settings=value,
                    program_name=value,
                    request_id=value,
                )

            Call from CLI:

            .. code-block:: bash

                $ idem exec aws_auto.medialive.multiplex.program.create multiplex_id=value, multiplex_program_settings=value, program_name=value, request_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="medialive",
        operation="create_multiplex_program",
        op_kwargs={
            "MultiplexId": multiplex_id,
            "MultiplexProgramSettings": multiplex_program_settings,
            "ProgramName": program_name,
            "RequestId": request_id,
        },
    )


async def delete(hub, ctx, multiplex_id: str, program_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Delete a program from a multiplex.

    Args:
        multiplex_id(str): The ID of the multiplex that the program belongs to.
        program_name(str): The multiplex program name.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.medialive.multiplex.program.delete(
                ctx, multiplex_id=value, program_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.medialive.multiplex.program.delete multiplex_id=value, program_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="medialive",
        operation="delete_multiplex_program",
        op_kwargs={"MultiplexId": multiplex_id, "ProgramName": program_name},
    )


async def describe(hub, ctx, multiplex_id: str, program_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Get the details for a program in a multiplex.

    Args:
        multiplex_id(str): The ID of the multiplex that the program belongs to.
        program_name(str): The name of the program.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.medialive.multiplex.program.describe(
                ctx, multiplex_id=value, program_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.medialive.multiplex.program.describe multiplex_id=value, program_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="medialive",
        operation="describe_multiplex_program",
        op_kwargs={"MultiplexId": multiplex_id, "ProgramName": program_name},
    )


async def list_all(
    hub, ctx, multiplex_id: str, max_results: int = None, next_token: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    List the programs that currently exist for a specific multiplex.

    Args:
        max_results(int, optional): The maximum number of items to return. Defaults to None.
        multiplex_id(str): The ID of the multiplex that the programs belong to.
        next_token(str, optional): The token to retrieve the next page of results. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.medialive.multiplex.program.list_all(ctx, multiplex_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.medialive.multiplex.program.list_all multiplex_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="medialive",
        operation="list_multiplex_programs",
        op_kwargs={
            "MaxResults": max_results,
            "MultiplexId": multiplex_id,
            "NextToken": next_token,
        },
    )


async def update(
    hub,
    ctx,
    multiplex_id: str,
    program_name: str,
    multiplex_program_settings: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Update a program in a multiplex.

    Args:
        multiplex_id(str): The ID of the multiplex of the program to update.
        multiplex_program_settings(Dict, optional): The new settings for a multiplex program. Defaults to None.
        program_name(str): The name of the program to update.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.medialive.multiplex.program.update(
                ctx, multiplex_id=value, program_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.medialive.multiplex.program.update multiplex_id=value, program_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="medialive",
        operation="update_multiplex_program",
        op_kwargs={
            "MultiplexId": multiplex_id,
            "MultiplexProgramSettings": multiplex_program_settings,
            "ProgramName": program_name,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def add(
    hub,
    ctx,
    communication_body: str,
    case_id: str = None,
    cc_email_addresses: List = None,
    attachment_set_id: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Adds additional customer communication to an AWS Support case. Use the caseId parameter to identify the case to which to add communication. You can list a set of email addresses to copy on the communication by using the ccEmailAddresses parameter. The communicationBody value contains the text of the communication.    You must have a Business or Enterprise Support plan to use the AWS Support API.    If you call the AWS Support API from an account that does not have a Business or Enterprise Support plan, the SubscriptionRequiredException error message appears. For information about changing your support plan, see AWS Support.

    Args:
        case_id(str, optional): The support case ID requested or returned in the call. The case ID is an alphanumeric string formatted as shown in this example: case-12345678910-2013-c4c1d2bf33c5cf47 . Defaults to None.
        communication_body(str): The body of an email communication to add to the support case.
        cc_email_addresses(List, optional): The email addresses in the CC line of an email to be added to the support case. Defaults to None.
        attachment_set_id(str, optional): The ID of a set of one or more attachments for the communication to add to the case. Create the set by calling AddAttachmentsToSet . Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.support.communication.to_case.add(ctx, communication_body=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.support.communication.to_case.add communication_body=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="support",
        operation="add_communication_to_case",
        op_kwargs={
            "caseId": case_id,
            "communicationBody": communication_body,
            "ccEmailAddresses": cc_email_addresses,
            "attachmentSetId": attachment_set_id,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(hub, ctx, rule_set_name: str, rule: Dict, after: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Creates a receipt rule. For information about setting up receipt rules, see the Amazon SES Developer Guide. You can execute this operation no more than once per second.

    Args:
        rule_set_name(str): The name of the rule set that the receipt rule will be added to.
        after(str, optional): The name of an existing rule after which the new rule will be placed. If this parameter is null, the new rule will be inserted at the beginning of the rule list. Defaults to None.
        rule(Dict): A data structure that contains the specified rule's name, actions, recipients, domains, enabled status, scan status, and TLS policy.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ses.receipt.rule.init.create(
                ctx, rule_set_name=value, rule=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ses.receipt.rule.init.create rule_set_name=value, rule=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ses",
        operation="create_receipt_rule",
        op_kwargs={"RuleSetName": rule_set_name, "After": after, "Rule": rule},
    )


async def delete(hub, ctx, rule_set_name: str, rule_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the specified receipt rule. For information about managing receipt rules, see the Amazon SES Developer Guide. You can execute this operation no more than once per second.

    Args:
        rule_set_name(str): The name of the receipt rule set that contains the receipt rule to delete.
        rule_name(str): The name of the receipt rule to delete.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ses.receipt.rule.init.delete(
                ctx, rule_set_name=value, rule_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ses.receipt.rule.init.delete rule_set_name=value, rule_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ses",
        operation="delete_receipt_rule",
        op_kwargs={"RuleSetName": rule_set_name, "RuleName": rule_name},
    )


async def describe(hub, ctx, rule_set_name: str, rule_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Returns the details of the specified receipt rule. For information about setting up receipt rules, see the Amazon SES Developer Guide. You can execute this operation no more than once per second.

    Args:
        rule_set_name(str): The name of the receipt rule set that the receipt rule belongs to.
        rule_name(str): The name of the receipt rule.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ses.receipt.rule.init.describe(
                ctx, rule_set_name=value, rule_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ses.receipt.rule.init.describe rule_set_name=value, rule_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ses",
        operation="describe_receipt_rule",
        op_kwargs={"RuleSetName": rule_set_name, "RuleName": rule_name},
    )


async def update(hub, ctx, rule_set_name: str, rule: Dict) -> Dict:
    r"""
    **Autogenerated function**

    Updates a receipt rule. For information about managing receipt rules, see the Amazon SES Developer Guide. You can execute this operation no more than once per second.

    Args:
        rule_set_name(str): The name of the receipt rule set that the receipt rule belongs to.
        rule(Dict): A data structure that contains the updated receipt rule information.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ses.receipt.rule.init.update(
                ctx, rule_set_name=value, rule=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ses.receipt.rule.init.update rule_set_name=value, rule=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ses",
        operation="update_receipt_rule",
        op_kwargs={"RuleSetName": rule_set_name, "Rule": rule},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def deregister(hub, ctx, baseline_id: str, patch_group: str) -> Dict:
    r"""
    **Autogenerated function**

    Removes a patch group from a patch baseline.

    Args:
        baseline_id(str): The ID of the patch baseline to deregister the patch group from.
        patch_group(str): The name of the patch group that should be deregistered from the patch baseline.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.patch.baseline.for_patch_group.deregister(
                ctx, baseline_id=value, patch_group=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.patch.baseline.for_patch_group.deregister baseline_id=value, patch_group=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="deregister_patch_baseline_for_patch_group",
        op_kwargs={"BaselineId": baseline_id, "PatchGroup": patch_group},
    )


async def get(hub, ctx, patch_group: str, operating_system: str = None) -> Dict:
    r"""
    **Autogenerated function**

    Retrieves the patch baseline that should be used for the specified patch group.

    Args:
        patch_group(str): The name of the patch group whose patch baseline should be retrieved.
        operating_system(str, optional): Returns he operating system rule specified for patch groups using the patch baseline. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.patch.baseline.for_patch_group.get(ctx, patch_group=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.patch.baseline.for_patch_group.get patch_group=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="get_patch_baseline_for_patch_group",
        op_kwargs={"PatchGroup": patch_group, "OperatingSystem": operating_system},
    )


async def register(hub, ctx, baseline_id: str, patch_group: str) -> Dict:
    r"""
    **Autogenerated function**

    Registers a patch baseline for a patch group.

    Args:
        baseline_id(str): The ID of the patch baseline to register with the patch group.
        patch_group(str): The name of the patch group to be registered with the patch baseline.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.patch.baseline.for_patch_group.register(
                ctx, baseline_id=value, patch_group=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.patch.baseline.for_patch_group.register baseline_id=value, patch_group=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="register_patch_baseline_for_patch_group",
        op_kwargs={"BaselineId": baseline_id, "PatchGroup": patch_group},
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


__virtualname__ = "principal_tag_attribute_map"


__func_alias__ = {"set_": "set"}


async def get(hub, ctx, identity_pool_id: str, identity_provider_name: str) -> Dict:
    r"""
    **Autogenerated function**

    Use GetPrincipalTagAttributeMap to list all mappings between PrincipalTags and user attributes.

    Args:
        identity_pool_id(str): You can use this operation to get the ID of the Identity Pool you setup attribute mappings for.
        identity_provider_name(str): You can use this operation to get the provider name.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_identity.principal_tag_attribute_map_.get(
                ctx, identity_pool_id=value, identity_provider_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_identity.principal_tag_attribute_map.get identity_pool_id=value, identity_provider_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-identity",
        operation="get_principal_tag_attribute_map",
        op_kwargs={
            "IdentityPoolId": identity_pool_id,
            "IdentityProviderName": identity_provider_name,
        },
    )


async def set_(
    hub,
    ctx,
    identity_pool_id: str,
    identity_provider_name: str,
    use_defaults: bool = None,
    principal_tags: Dict = None,
) -> Dict:
    r"""
    **Autogenerated function**

    You can use this operation to use default (username and clientID) attribute or custom attribute mappings.

    Args:
        identity_pool_id(str): The ID of the Identity Pool you want to set attribute mappings for.
        identity_provider_name(str): The provider name you want to use for attribute mappings.
        use_defaults(bool, optional): You can use this operation to use default (username and clientID) attribute mappings. Defaults to None.
        principal_tags(Dict, optional): You can use this operation to add principal tags. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cognito_identity.principal_tag_attribute_map_.set(
                ctx, identity_pool_id=value, identity_provider_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cognito_identity.principal_tag_attribute_map.set identity_pool_id=value, identity_provider_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cognito-identity",
        operation="set_principal_tag_attribute_map",
        op_kwargs={
            "IdentityPoolId": identity_pool_id,
            "IdentityProviderName": identity_provider_name,
            "UseDefaults": use_defaults,
            "PrincipalTags": principal_tags,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def create(
    hub,
    ctx,
    s3_url_signer_role: str,
    software_to_update: str,
    update_targets: List,
    update_targets_architecture: str,
    update_targets_operating_system: str,
    amzn_client_token: str = None,
    update_agent_log_level: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Creates a software update for a core or group of cores (specified as an IoT thing group.) Use this to update the OTA Agent as well as the Greengrass core software. It makes use of the IoT Jobs feature which provides additional commands to manage a Greengrass core software update job.

    Args:
        amzn_client_token(str, optional): A client token used to correlate requests and responses. Defaults to None.
        s3_url_signer_role(str): The IAM Role that Greengrass will use to create pre-signed URLs pointing towards the update artifact.
        software_to_update(str): The piece of software on the Greengrass core that will be updated.
        update_agent_log_level(str, optional): The minimum level of log statements that should be logged by the OTA Agent during an update. Defaults to None.
        update_targets(List): The ARNs of the targets (IoT things or IoT thing groups) that this update will be applied to.
        update_targets_architecture(str): The architecture of the cores which are the targets of an update.
        update_targets_operating_system(str): The operating system of the cores which are the targets of an update.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.greengrass.software_update_job.create(
                ctx,
                s3_url_signer_role=value,
                software_to_update=value,
                update_targets=value,
                update_targets_architecture=value,
                update_targets_operating_system=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.greengrass.software_update_job.create s3_url_signer_role=value, software_to_update=value, update_targets=value, update_targets_architecture=value, update_targets_operating_system=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="greengrass",
        operation="create_software_update_job",
        op_kwargs={
            "AmznClientToken": amzn_client_token,
            "S3UrlSignerRole": s3_url_signer_role,
            "SoftwareToUpdate": software_to_update,
            "UpdateAgentLogLevel": update_agent_log_level,
            "UpdateTargets": update_targets,
            "UpdateTargetsArchitecture": update_targets_architecture,
            "UpdateTargetsOperatingSystem": update_targets_operating_system,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict
from typing import List


async def associate(
    hub,
    ctx,
    ops_item_id: str,
    association_type: str,
    resource_type: str,
    resource_uri: str,
) -> Dict:
    r"""
    **Autogenerated function**

    Associates a related resource to a Systems Manager OpsCenter OpsItem. For example, you can associate an Incident Manager incident or analysis with an OpsItem. Incident Manager is a capability of Amazon Web Services Systems Manager.

    Args:
        ops_item_id(str): The ID of the OpsItem to which you want to associate a resource as a related item.
        association_type(str): The type of association that you want to create between an OpsItem and a resource. OpsCenter supports IsParentOf and RelatesTo association types.
        resource_type(str): The type of resource that you want to associate with an OpsItem. OpsCenter supports the following types:  AWS::SSMIncidents::IncidentRecord: an Incident Manager incident. Incident Manager is a capability of Amazon Web Services Systems Manager.  AWS::SSM::Document: a Systems Manager (SSM) document.
        resource_uri(str): The Amazon Resource Name (ARN) of the Amazon Web Services resource that you want to associate with the OpsItem.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.ops.item.related_item.associate(
                ctx,
                ops_item_id=value,
                association_type=value,
                resource_type=value,
                resource_uri=value,
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.ops.item.related_item.associate ops_item_id=value, association_type=value, resource_type=value, resource_uri=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="associate_ops_item_related_item",
        op_kwargs={
            "OpsItemId": ops_item_id,
            "AssociationType": association_type,
            "ResourceType": resource_type,
            "ResourceUri": resource_uri,
        },
    )


async def disassociate(hub, ctx, ops_item_id: str, association_id: str) -> Dict:
    r"""
    **Autogenerated function**

    Deletes the association between an OpsItem and a related resource. For example, this API operation can delete an Incident Manager incident from an OpsItem. Incident Manager is a capability of Amazon Web Services Systems Manager.

    Args:
        ops_item_id(str): The ID of the OpsItem for which you want to delete an association between the OpsItem and a related resource.
        association_id(str): The ID of the association for which you want to delete an association between the OpsItem and a related resource.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.ops.item.related_item.disassociate(
                ctx, ops_item_id=value, association_id=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.ops.item.related_item.disassociate ops_item_id=value, association_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="disassociate_ops_item_related_item",
        op_kwargs={"OpsItemId": ops_item_id, "AssociationId": association_id},
    )


async def list_all(
    hub,
    ctx,
    ops_item_id: str = None,
    filters: List = None,
    max_results: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Lists all related-item resources associated with an OpsItem.

    Args:
        ops_item_id(str, optional): The ID of the OpsItem for which you want to list all related-item resources. Defaults to None.
        filters(List, optional): One or more OpsItem filters. Use a filter to return a more specific list of results. . Defaults to None.
        max_results(int, optional): The maximum number of items to return for this call. The call also returns a token that you can specify in a subsequent call to get the next set of results. Defaults to None.
        next_token(str, optional): The token for the next set of items to return. (You received this token from a previous call.). Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.ssm.ops.item.related_item.list_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.ssm.ops.item.related_item.list_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="ssm",
        operation="list_ops_item_related_items",
        op_kwargs={
            "OpsItemId": ops_item_id,
            "Filters": filters,
            "MaxResults": max_results,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe(
    hub,
    ctx,
    conformance_pack_name: str,
    filters: Dict = None,
    limit: int = None,
    next_token: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns compliance details for each rule in that conformance pack.  You must provide exact rule names.

    Args:
        conformance_pack_name(str): Name of the conformance pack.
        filters(Dict, optional): A ConformancePackComplianceFilters object. Defaults to None.
        limit(int, optional): The maximum number of AWS Config rules within a conformance pack are returned on each page. Defaults to None.
        next_token(str, optional): The nextToken string returned in a previous request that you use to request the next page of results in a paginated response. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.config.conformance_pack.compliance.init.describe(
                ctx, conformance_pack_name=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.config.conformance_pack.compliance.init.describe conformance_pack_name=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="config",
        operation="describe_conformance_pack_compliance",
        op_kwargs={
            "ConformancePackName": conformance_pack_name,
            "Filters": filters,
            "Limit": limit,
            "NextToken": next_token,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def describe_all(
    hub,
    ctx,
    cluster_identifier: str = None,
    max_records: int = None,
    marker: str = None,
) -> Dict:
    r"""
    **Autogenerated function**

    Returns an array of ClusterDbRevision objects.

    Args:
        cluster_identifier(str, optional): A unique identifier for a cluster whose ClusterDbRevisions you are requesting. This parameter is case sensitive. All clusters defined for an account are returned by default. Defaults to None.
        max_records(int, optional): The maximum number of response records to return in each call. If the number of remaining response records exceeds the specified MaxRecords value, a value is returned in the marker field of the response. You can retrieve the next set of response records by providing the returned marker value in the marker parameter and retrying the request.  Default: 100 Constraints: minimum 20, maximum 100. Defaults to None.
        marker(str, optional): An optional parameter that specifies the starting point for returning a set of response records. When the results of a DescribeClusterDbRevisions request exceed the value specified in MaxRecords, Amazon Redshift returns a value in the marker field of the response. You can retrieve the next set of response records by providing the returned marker value in the marker parameter and retrying the request.  Constraints: You can specify either the ClusterIdentifier parameter, or the marker parameter, but not both. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.redshift.cluster.db_revision.describe_all(ctx)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.redshift.cluster.db_revision.describe_all
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="redshift",
        operation="describe_cluster_db_revisions",
        op_kwargs={
            "ClusterIdentifier": cluster_identifier,
            "MaxRecords": max_records,
            "Marker": marker,
        },
    )


async def modify(hub, ctx, cluster_identifier: str, revision_target: str) -> Dict:
    r"""
    **Autogenerated function**

    Modifies the database revision of a cluster. The database revision is a unique revision of the database running in a cluster.

    Args:
        cluster_identifier(str): The unique identifier of a cluster whose database revision you want to modify.  Example: examplecluster .
        revision_target(str): The identifier of the database revision. You can retrieve this value from the response to the DescribeClusterDbRevisions request.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.redshift.cluster.db_revision.modify(
                ctx, cluster_identifier=value, revision_target=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.redshift.cluster.db_revision.modify cluster_identifier=value, revision_target=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="redshift",
        operation="modify_cluster_db_revision",
        op_kwargs={
            "ClusterIdentifier": cluster_identifier,
            "RevisionTarget": revision_target,
        },
    )



"""
Autogenerated module using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__
"""
from typing import Dict


async def create(hub, ctx, distribution_id: str, invalidation_batch: Dict) -> Dict:
    r"""
    **Autogenerated function**

    Create a new invalidation.

    Args:
        distribution_id(str): The distribution's id.
        invalidation_batch(Dict): The batch information for the invalidation.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.invalidation.create(
                ctx, distribution_id=value, invalidation_batch=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.invalidation.create distribution_id=value, invalidation_batch=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="create_invalidation",
        op_kwargs={
            "DistributionId": distribution_id,
            "InvalidationBatch": invalidation_batch,
        },
    )


async def get(hub, ctx, distribution_id: str, id_: str) -> Dict:
    r"""
    **Autogenerated function**

    Get the information about an invalidation.

    Args:
        distribution_id(str): The distribution's ID.
        id_(str): The identifier for the invalidation request, for example, IDFDVBD632BHDS5.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.invalidation.get(
                ctx, distribution_id=value, id_=value
            )

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.invalidation.get distribution_id=value, id_=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="get_invalidation",
        op_kwargs={"DistributionId": distribution_id, "Id": id_},
    )


async def list_all(
    hub, ctx, distribution_id: str, marker: str = None, max_items: str = None
) -> Dict:
    r"""
    **Autogenerated function**

    Lists invalidation batches.

    Args:
        distribution_id(str): The distribution's ID.
        marker(str, optional): Use this parameter when paginating results to indicate where to begin in your list of invalidation batches. Because the results are returned in decreasing order from most recent to oldest, the most recent results are on the first page, the second page will contain earlier results, and so on. To get the next page of results, set Marker to the value of the NextMarker from the current page's response. This value is the same as the ID of the last invalidation batch on that page. . Defaults to None.
        max_items(str, optional): The maximum number of invalidation batches that you want in the response body. Defaults to None.

    Returns:
        Dict

    Examples:
        Call from code:

        .. code-block:: python

            await hub.exec.aws_auto.cloudfront.invalidation.list_all(ctx, distribution_id=value)

        Call from CLI:

        .. code-block:: bash

            $ idem exec aws_auto.cloudfront.invalidation.list_all distribution_id=value
    """
    return await hub.tool.aws.client.exec(
        ctx,
        service_name="cloudfront",
        operation="list_invalidations",
        op_kwargs={
            "DistributionId": distribution_id,
            "Marker": marker,
            "MaxItems": max_items,
        },
    )
